/*  Copyright 2000-2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _GEOBASE_H_INCLUDED_
#define _GEOBASE_H_INCLUDED_

#include "Plane.h"

typedef int TType;

#define tpPoint 1
#define tpLine 2
#define tpCircle 4
#define tpAngle 8
#define tpFixPoint 16
#define tpComment 32
#define tpNorm 64
#define tpVector 128

class GeoBase {

 public:
  GeoBase( Plan* pln, int idsn );
  virtual ~GeoBase();
  void	DeleteAll();
  
  virtual bool		Inside( Vect& scrnpoint ) = 0;
  void		Paint();
  virtual void		DoPaint() = 0;
  virtual GeoBase*	Event( TEvent event, Vect& scrnpoint );
  
  void		InvalidateAllOfThem();
  virtual void		DoDefine() = 0;
  void		Define();
  void		DefineAll();
  
  virtual void		DoStore( CMyArchive& ar );
  void		Store( CMyArchive& ar );
  
  virtual	void		DoCheckDep() = 0;
  void		CheckDep();
  
  void		DrawLetter( Vect& );
  
  void		Select();	 // Select this object ( bZone = extend to children )
  void		SelectAll();
  void		ChgSelect();
  virtual void		SelectFromParents();
  void		UnSelect( bool bAll );	// Unselect ( bAll = Unselect every single object )		
  void		ExtendSelectToZone();
  void		SelectFromZone( Vect p1, Vect p2 );
  virtual void		DoSelectFromZone( Vect p1, Vect p2 );
  void          DispatchNewLetter( char letter );
  
  virtual Vect		Rectify( Vect& p );	// Get this point back inside the object
  virtual bool		bNeedPoint();
  
  void		Destroy();
  void		BurnTheDeads();
  void		UnMark();
  void		ctSelected();
  void		DeleteSelected();
  void          HideSelected();
  GeoBase*	GetSelected();
  
  void		SetIndex( int i = 0 );
  GeoBase*	GetBaseFromIndex( int index );
  virtual void	DoLinkFromIndex();
  void		LinkFromIndex();
  
  virtual void		Properties();	// Rponse  un click sur Properties d'un menu local
  // Crer une boite ( minimum : nom de l'objet )
  void		InvisibleSelected();
  virtual void		Left() {}
  virtual void		Right() {}
  virtual void		Up() {}
  virtual void		Down() {}
  
  bool	bExists; // TRUE = this object successfuly defined itself (can be FALSE for instance 
           // if this object depends on the intersection of two parallel lines)
  bool	bDestroying; 
  TType	Type;
  bool	bMark;   // a marked object is an object currently used for the definition of a new object
  bool	bVisible;    // Set this to FALSE inhibates methods DoPaint() and Inside()
  bool	bDestroyable; // currently unused : TRUE = this object cannot be destroyed
  int	Index; // index of this object for serialized reference during storage in an archive
  
  // -- Building phase
  
  virtual bool		AddObject( GeoBase* obj ) = 0;
  // return TRUE ie build is done
  virtual bool		AddPoint( Vect& scrnpoint ) = 0;
  // idm ... 
  
  Plan*	Pln;
  int     Col;
  int		Width;
  int		Style;
  mlString	Name;
  char	Letter[3];
  
  bool	Status; // FALSE = i'm currently building myself - TRUE = i'm a wellformed and built object
  
  void	BuildDone(); // Insert ourselves in the scene since we are built
  
  bool	bNoStore;
  bool	bPaintOver;
  bool	bSelect;  // i'm a selected object ?
  bool	bDefined; // Yet, am I defined in the current redefinition process ?
  GeoBase* Precedent, *Suivant;
};

class GeoPoint : public GeoBase {

public:
	GeoPoint( Plan* pln, int idsn );
	virtual bool		Inside( Vect& scrnpoint );
	virtual void		DoPaint();
	virtual void		DoSelectFromZone( Vect p1, Vect p2 );

	Vect	p;
	bool	bShift;
};

class GeoLine : public GeoBase {

public:
	GeoLine( Plan* pln, int idsn );
	virtual bool		Inside( Vect& scrnpoint );
	virtual	Vect		Rectify( Vect& p );	// Ramener le point  l'intrieur de l'objet
	virtual void		DoPaint();

	Vect	o, u;		// Origine, vecteur directeur.
};

class GeoCircle : public GeoBase {

public:
	GeoCircle( Plan* pln, int idsn );
	virtual bool		Inside( Vect& scrnpoint );
	virtual	Vect		Rectify( Vect& p );	// Ramener le point  l'intrieur de l'objet
	virtual void		DoPaint();

	Vect	o;
	double  r; // Origine, rayon
};

class GeoAngle : public GeoBase {

public:
	GeoAngle( Plan* pln );
	GeoAngle( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoAngle( Plan *pln, GeoPoint*, GeoPoint*, GeoPoint* );

	virtual bool		Inside( Vect& scrnpoint );
	virtual void		DoPaint();
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		SelectFromParents();

	bool		bOriented;
protected:
	bool		bShift;
	double          alpha;
	bool            bClockwise;
	Vect		o, p1, p2;
	GeoPoint	*_o, *_p1, *_p2;
	int		IO, I1, I2;
	int		nPoints;
};

class GeoFixPoint : public GeoPoint {

public:
	GeoFixPoint( Plan *pln, bool bLimited = FALSE );
	GeoFixPoint( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoFixPoint( Plan *pln, double x, double y );

	virtual GeoBase*	Event( TEvent event, Vect& scrnpoint );

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoLinkFromIndex();
	virtual void		DoStore( CMyArchive& ar );
	void		isRestricted( GeoBase* geo );
	virtual void            Properties();	      
	virtual void		Left();
	virtual void		Right();
	virtual void		Up();
	virtual void		Down();

	bool		bRestrict;

protected:
	bool	bSliding;
	int		xSlide, ySlide;

	GeoBase*	Geo;
	int			IGeo;
};

class GeoPointIntersect : public GeoPoint {

public:
	GeoPointIntersect( Plan *pln );
	GeoPointIntersect( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoPointIntersect( Plan *pln, GeoLine*, GeoLine* );

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();

protected:
	GeoLine *d1, *d2;
	int		I1, I2;
	int		nLines;
};

class GeoPointRotated : public GeoPoint {

public:
	GeoPointRotated( Plan *pln );
	GeoPointRotated( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoPointRotated( Plan *pln, GeoPoint*, GeoPoint*, double );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
			void		GetAngle();
			void		Properties();

protected:
	GeoPoint *p1, *p2;
	int		I1, I2;
	double	Alpha, Factor;	// Angle de la rotation et facteur de l'homothtie
	int		nPoints;
};

class GeoMidPoint : public GeoPoint {

public:
	GeoMidPoint( Plan *pln );
	GeoMidPoint( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoMidPoint( Plan *pln, GeoPoint*, GeoPoint* );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();

protected:
	GeoPoint *p1, *p2;
	int		I1, I2;
	int		nPoints;
};

class GeoCarre : public GeoPoint {

public:
	GeoCarre( Plan *pln );
	GeoCarre( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoCarre( Plan *pln, GeoPoint*, GeoPoint* );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		SelectFromParents();

protected:
	GeoPoint *p1, *p2;
	int		I1, I2;
	int		nPoints;
};

class GeoVect : public GeoPoint {

public:
	GeoVect( Plan *pln );
	GeoVect( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoVect( Plan *pln, GeoPoint*, GeoPoint*, GeoPoint* );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();

protected:
	GeoPoint *p1, *p2, *p3;
	int		I1, I2, I3;
	int		nPoints;
};

class GeoLineAB : public GeoLine {

public:
	GeoLineAB( Plan *pln );
	GeoLineAB( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoLineAB( Plan *pln, GeoPoint*, GeoPoint*, bool );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();
	virtual void		DoPaint();	// Surcharges pour le cas des segments 
	virtual bool		Inside( Vect& ); // -----	
	virtual	Vect		Rectify( Vect& p );	// Ramener le point  l'intrieur de l'objet


	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
			void		IsSegment();
			void		Properties();
	virtual void		SelectFromParents();

	bool	bSegment;	// Est-ce un segment ?

protected:
	GeoPoint *p1, *p2;
	int		I1, I2;
	int		nPoints;
};

class GeoOrthogonal : public GeoLine {

public:
	GeoOrthogonal( Plan *pln );
	GeoOrthogonal( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoOrthogonal( Plan *pln, GeoPoint*, GeoLine* );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		SelectFromParents();

protected:
	GeoPoint *p; 
	GeoLine  *d;
	int		I1, I2;
	bool	pDef;
	bool	dDef;
};

class GeoParallel : public GeoLine {

public:
	GeoParallel( Plan *pln );
	GeoParallel( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoParallel( Plan *pln, GeoPoint*, GeoLine* );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		SelectFromParents();

protected:
	GeoPoint *p; 
	GeoLine  *d;
	int		I1, I2;
	bool	pDef;
	bool	dDef;
};

class GeoCircleAB : public GeoCircle {

public:
	GeoCircleAB( Plan *pln );
	GeoCircleAB( Plan *pln, int index, GeoBase* racine, CMyArchive& ar );
	GeoCircleAB( Plan *pln, GeoPoint*, GeoPoint* );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		SelectFromParents();

protected:
	GeoPoint *p1, *p2; 
	int		I1, I2;
	int nPoints;
};

class GeoProjOrthogonal : public GeoPoint {

public:
	GeoProjOrthogonal( Plan *pln, bool _bSymetrique = FALSE );
	GeoProjOrthogonal( Plan *pln, int index, GeoBase* racine, CMyArchive& ar, bool _bSymetrique = FALSE );
	GeoProjOrthogonal( Plan *pln, GeoPoint*, GeoLine* );
	virtual bool		bNeedPoint();

	virtual void		DoDefine();
	virtual	void		DoCheckDep();

	virtual bool		AddObject( GeoBase* obj );
		// return TRUE ie construction acheve.
	virtual bool		AddPoint( Vect& scrnpoint );
		// idm ... 
	virtual void		DoStore( CMyArchive& ar );
	virtual void		DoLinkFromIndex();
	virtual void		SelectFromParents();

	bool	bSymetrique;	
protected:
	GeoPoint *p; 
	GeoLine  *d;
	int		I1, I2;
	bool	pDef;
	bool	dDef;
};

#endif // !defined(AFX_GEOBASE_H__F6792DAC_9444_11D3_9C74_9160B3526B31__INCLUDED_)
