/*  Copyright 2000-2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "id.h"
#include "GeoComment.h"
#include "euclideView.h"
#include "euclideDocument.h"
#include "diaBase.h"

GeoComment::GeoComment( Plan* pln, int idsn ):GeoBase( pln, idsn ) {

	bPaintOver = TRUE;
	String = mlString( IDS_COMMENT_DEFAULT );
	Type = tpComment;
	nPoints = 0;
	Status = FALSE;
	bSliding = FALSE;
	Suivant = NULL;
}

GeoComment::GeoComment( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ):GeoBase( pln, IDSN_COMMENT ) {

	bPaintOver = TRUE;
	Type = tpComment;
	nPoints = 0;
	Index = index;
	ar >> bVisible >> Col >> p1.X[0] >> p1.X[1] >> String;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	bSliding = FALSE;
	Status = TRUE;
}

bool GeoComment::Inside( Vect& scrnpoint ) {

	if ( !bVisible ) return FALSE;

	Vect _p1 = Pln->GetScreenCoords( p1 );
	Vect _p2 = _p1 + Vect(30,10,0);

	return (( _p1.X[0] < scrnpoint.X[0] )&&( _p2.X[0] > scrnpoint.X[0] )
		&&( _p1.X[1] < scrnpoint.X[1] )&&( _p2.X[1] > scrnpoint.X[1] ));
}

void GeoComment::DoPaint() {

	Vect _p1 = Pln->GetScreenCoords( p1 );

	int p1x = (int)_p1.X[0];	int p1y = (int)_p1.X[1];

	Pln->getDoc()->paintComment( p1x, p1y, Col, String, bSelect );
}

void GeoComment::DoDefine() {

	bExists = TRUE;
}

void GeoComment::DoCheckDep() {}

bool GeoComment::AddObject( GeoBase* obj ) {

	return FALSE;
}

bool GeoComment::AddPoint( Vect& scrnpoint ) {

	p1 = Pln->GetMathCoords( scrnpoint );

	BuildDone();
	return TRUE;
}

void GeoComment::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_COMMENT );
  ar << bVisible << Col << p1.X[0] << p1.X[1] << String;
}

void GeoComment::Properties() {

  Pln->StoreUndo();
  diaComment dia( Name, String );
  String = dia.run();
  bSliding = FALSE;
}

void GeoComment::DoLinkFromIndex() {}

GeoBase* GeoComment::Event( TEvent event, Vect& scrnpoint ) {
  
  switch( event ) {
    
  case evLMouse:
    if ( Inside( scrnpoint ) ) {
      
      bSliding = 3;
      p1Sliding = p1 - Pln->GetMathCoords( scrnpoint );
      if ( ! Pln->Status ) view->setMessageText( Name );
     return this;
    }
    break;
  case evLMouseUp:
    bSliding = FALSE;
    break;
    
  case evMouseMove:
    if ( bSliding ) {
      
      Vect p = Pln->GetMathCoords( 
				  Vect( scrnpoint.X[0],
					scrnpoint.X[1],
					0 ) );
      if ( bSliding == 3 ) {
	p1 = p1Sliding + p;
      }
      Pln->InvalidateAllOfThem();
      return this;
    }
    break;
  case evRMouse:
  case evRMouseUp:
  case evLMouseDblClk:
    break;
  }
  
  return GeoBase::Event( event, scrnpoint );
}

void GeoComment::DoSelectFromZone( Vect _p1, Vect _p2 ) {

  Vect p0 = Pln->GetDcCoords( p1 );
  
  if (( p0.X[0] - _p1.X[0] )*( p0.X[0] - _p2.X[0] ) < 0 ) 
    if ((  p0.X[1] - _p1.X[1] )*( p0.X[1] - _p2.X[1] ) < 0 ) 
      bSelect = TRUE;
}

///////////////////////////////////////////////////////////////////
// GeoProjCercle -------------------------------------------------
// Central projection on a circle --------------------------------

GeoProjCercle::GeoProjCercle( Plan *pln ) : GeoPoint( pln, IDSN_PROJ_CIRCLE ) {

	bp = bc = FALSE;
}

GeoProjCercle::GeoProjCercle( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoPoint( pln, IDSN_PROJ_CIRCLE ) {

	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> Ip >> Ic;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoProjCercle::GeoProjCercle( Plan *pln, GeoPoint *_p, GeoCircle *_c ):GeoPoint( pln, IDSN_PROJ_CIRCLE ) {

	p = _p; c = _c;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoProjCercle::DoDefine() {

	if ( p ) p->Define();
	if ( c ) c->Define();
	if (( p->bExists )&&( c->bExists )) {

		Vect o = c->o;
		Vect p0 = p->p;
		double r = c->r;
		Vect u = p0 - o;

		GeoPoint::p = o + r*!u;
		
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoProjCercle::DoCheckDep() {

	if (( p->bDestroying )||( c->bDestroying ))
		Destroy();
}

bool GeoProjCercle::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

			p = (GeoPoint*)obj;
			bp = TRUE;
			obj->bMark = TRUE;
		}
	if ( obj->Type & tpCircle ) {

			c = (GeoCircle*)obj;
			bc = TRUE;
			obj->bMark = TRUE;
		}
	if ( bp && bc ) {
	
		BuildDone();
		return TRUE;
	}
	return FALSE;
}

bool GeoProjCercle::bNeedPoint() {

	return !bp;
}

bool GeoProjCercle::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoProjCercle::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_PROJ_CIRCLE ); 
  ar << bVisible << Letter[0] << Letter[1] << Col << p->Index << c->Index;
}

void GeoProjCercle::DoLinkFromIndex() {

	p = (GeoPoint*)Pln->Racine->GetBaseFromIndex( Ip );
	c = (GeoCircle*)Pln->Racine->GetBaseFromIndex( Ic );
}

//////////////////////////////////////////////////////////////////
// GeoBissectrice ------------------------------------------------
// Three points : center + 2 vectors ----------------------------

GeoBissectrice::GeoBissectrice( Plan *pln ) : GeoLine( pln, IDSN_BISSECTRICE ) {

	nPoints = 0;
}

GeoBissectrice::GeoBissectrice( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoLine( pln, IDSN_BISSECTRICE ) {

	Index = index;
	ar >> Width >> Style >> bVisible >> Col >> I1 >> I2 >> I3;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoBissectrice::GeoBissectrice( Plan *pln, GeoPoint *_p1, GeoPoint *_p2, GeoPoint *_p3 ):GeoLine( pln, IDSN_BISSECTRICE ) {

	p1 = _p1; p2 = _p2; p3 = _p3;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoBissectrice::DoDefine() {

	if ( p1 ) p1->Define();
	if ( p2 ) p2->Define();
	if ( p3 ) p3->Define();
	if (( p1->bExists )&&( p2->bExists )&&( p3->bExists )) {

		Vect o = p1->p;
		Vect u = p2->p - o;
		Vect v = p3->p - o;

		GeoLine::o = o;
		GeoLine::u = !( !u + !v );
		
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoBissectrice::DoCheckDep() {

	if (( p1->bDestroying )||( p2->bDestroying )||( p3->bDestroying ))
		Destroy();
}

bool GeoBissectrice::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

		if ( nPoints == 0 ) {
			p1 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 1 )&&( p1 != obj )) {
			p2 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 2 )&&( p2 != obj )&&( p1 != obj )) {
			p3 = (GeoPoint*)obj;
			BuildDone();
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoBissectrice::bNeedPoint() {

	return TRUE;
}

bool GeoBissectrice::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoBissectrice::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_BISSECTRICE );
  ar << Width << Style << bVisible << Col << p1->Index << p2->Index << p3->Index;
}

void GeoBissectrice::DoLinkFromIndex() {

	p1 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I1 );
	p2 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
	p3 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I3 );
}

void GeoBissectrice::SelectFromParents() {

	if ( p1->bSelect ) bSelect=TRUE;

	GeoLine::SelectFromParents();
}

///////////////////////////////////////////////////////////////////
// GeoInterCercleDroite -------------------------------------------
// Intersection of a line and a circle ----------------------------

GeoInterCercleDroite::GeoInterCercleDroite( Plan *pln ) : GeoPoint( pln, IDSN_INTER_CERCLE_DROITE ) {

	bl = bc = FALSE;
	bSlave = FALSE;
}

GeoInterCercleDroite::GeoInterCercleDroite( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) 
  : GeoPoint( pln, IDSN_INTER_CERCLE_DROITE) {

	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> bSlave >> Il >> Ic;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoInterCercleDroite::GeoInterCercleDroite( Plan *pln, GeoLine *_l, GeoCircle *_c, bool _bSlave )
  : GeoPoint( pln, IDSN_INTER_CERCLE_DROITE) {

	l = _l; c = _c;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
	bSlave = _bSlave;

	if ( !bSlave ) new GeoInterCercleDroite( pln, _l, _c, TRUE );
}

void GeoInterCercleDroite::DoDefine() {

	if ( l ) l->Define();
	if ( c ) c->Define();
	if (( l->bExists )&&( c->bExists )) {

		Vect u = l->u;	
		Vect op = c->o - l->o;
		double r = c->r;

		double A = u|u;
		double B = -2 * ( u|op );
		double C = ( op|op ) - r*r;
		double Delta = B*B - 4*A*C;

		if ( Delta < 0 ) {
			bExists = FALSE;
			return;
		}

		double t = ( -B + ((bSlave)?(1):(-1)) * sqrt(Delta) )/( 2*A );

		p = l->o + t * l->u;

		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoInterCercleDroite::DoCheckDep() {

	if (( l->bDestroying )||( c->bDestroying ))
		Destroy();
}

bool GeoInterCercleDroite::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpLine ) {

			l = (GeoLine*)obj;
			bl = TRUE;
			obj->bMark = TRUE;
		}
	if ( obj->Type & tpCircle ) {

			c = (GeoCircle*)obj;
			bc = TRUE;
			obj->bMark = TRUE;
		}
	if ( bl && bc ) {

		BuildDone();
		new GeoInterCercleDroite( Pln, l, c, TRUE );
		return TRUE;
	}
	return FALSE;
}

bool GeoInterCercleDroite::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoInterCercleDroite::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_INTER_CERCLE_DROITE );
  ar << bVisible << Letter[0] << Letter[1] << Col << bSlave << l->Index << c->Index;
}

void GeoInterCercleDroite::DoLinkFromIndex() {

	l = (GeoLine*)Pln->Racine->GetBaseFromIndex( Il );
	c = (GeoCircle*)Pln->Racine->GetBaseFromIndex( Ic );
}

///////////////////////////////////////////////////////////////////
// GeoInterCercleCercle -------------------------------------------
// Intersection of 2 circles --------------------------------------

GeoInterCercleCercle::GeoInterCercleCercle( Plan *pln ) : GeoPoint( pln, IDSN_INTER_CERCLE_CERCLE) {

	nCercle = 0;
	bSlave = FALSE;
}

GeoInterCercleCercle::GeoInterCercleCercle( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) 
  : GeoPoint( pln, IDSN_INTER_CERCLE_CERCLE) {

	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> bSlave >> I1 >> I2;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoInterCercleCercle::GeoInterCercleCercle( Plan *pln, GeoCircle *_c1, GeoCircle *_c2, bool _bSlave )
  :GeoPoint( pln, IDSN_INTER_CERCLE_CERCLE) {

	c1 = _c1; c2 = _c2;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
	bSlave = _bSlave;

	if ( !bSlave ) new GeoInterCercleCercle( pln, _c1, _c2, TRUE );
}

void GeoInterCercleCercle::DoDefine() {

	if ( c1 ) c1->Define();
	if ( c2 ) c2->Define();
	if (( c1->bExists )&&( c2->bExists )) {

		Vect o1		= c1->o;			double r1 = c1->r;
		Vect o2		= c2->o;			double r2 = c2->r;
		double L	= ( o2-o1 ).norm();

		if ( L < 1e-20 ) {
			bExists = FALSE;
			return;
		}
		double u2	= - ( r1*r1 - r2*r2 - L*L )/( 2*L );
		double u1	= L - u2;

		if ( u1 > r1 ) {
			bExists = FALSE;
			return;
		}

		double h	= (( bSlave )?( +1 ):( -1 )) * sqrt( r1*r1 - u1*u1 );

		p = o2 - u2 * (!( o2-o1 )) + h * (!(( o2-o1 )*Vect(0,0,1)));
		
		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoInterCercleCercle::DoCheckDep() {

	if (( c1->bDestroying )||( c2->bDestroying ))
		Destroy();
}

bool GeoInterCercleCercle::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpCircle ) {

		if ( nCercle == 0 ) {
			c1 = (GeoCircle*)obj;
			nCercle++;
			obj->bMark = TRUE;
		}
		if (( nCercle == 1 )&&( obj != c1 )) {
			c2 = (GeoCircle*)obj;
			obj->bMark = TRUE;			
			BuildDone();
			new GeoInterCercleCercle( Pln, c1, c2, TRUE );
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoInterCercleCercle::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoInterCercleCercle::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_INTER_CERCLE_CERCLE );
  ar << bVisible << Letter[0] << Letter[1] << Col << bSlave << c1->Index << c2->Index;
}

void GeoInterCercleCercle::DoLinkFromIndex() {

	c1 = (GeoCircle*)Pln->Racine->GetBaseFromIndex( I1 );
	c2 = (GeoCircle*)Pln->Racine->GetBaseFromIndex( I2 );
}

void GeoInterCercleCercle::SelectFromParents() {

	if (( c1->bSelect )||( c2->bSelect )) bSelect = TRUE;
}

///////////////////////////////////////////////////////////////////
// GeoBary --------------------------------------------------------
// Barycentre d'au maximum 6 points -------------------------------

GeoBary::GeoBary( Plan *pln ) : GeoPoint( pln, IDSN_BARY ) {

  nPoints = 2;
  for ( int i=0 ; i<6 ; i++ ) m[i] = 1.0;
  diaBary dia( Name, Letter, TRUE, nPoints, m );
  dia.run();
  nPoints = dia.getNPoints();
  for ( int i = 0 ; i < 6 ; i++ ) m[i] = dia.getWeight(i);
  
  iPoints = 0;
}

GeoBary::GeoBary( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoPoint( pln, IDSN_BARY ) {

	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> nPoints;
	switch ( nPoints ) {	
	case 6 :
		ar >> I[5];
	case 5 :
		ar >> I[4];
	case 4 :
		ar >> I[3];
	case 3 :
		ar >> I[2];
	case 2 :
		ar >> I[1];
	case 1 :
		ar >> I[0];
	}
	ar >> m[0] >> m[1] >> m[2] >> m[3] >> m[4] >> m[5];

	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

void GeoBary::DoDefine() {

	bExists = TRUE;
	double Alpha = 0;	// Somme des poids
	switch ( nPoints ) {

	case 6:
		if ( p[5] ) p[5]->Define();
		if ( !p[5]->bExists ) bExists = FALSE;
		Alpha += m[5];
	case 5:
		if ( p[4] ) p[4]->Define();
		if ( !p[4]->bExists ) bExists = FALSE;
		Alpha += m[4];
	case 4:
		if ( p[3] ) p[3]->Define();
		if ( !p[3]->bExists ) bExists = FALSE;
		Alpha += m[3];
	case 3:
		if ( p[2] ) p[2]->Define();
		if ( !p[2]->bExists ) bExists = FALSE;
		Alpha += m[2];
	case 2:
		if ( p[1] ) p[1]->Define();
		if ( !p[1]->bExists ) bExists = FALSE;
		Alpha += m[1];
	case 1:
		if ( p[0] ) p[0]->Define();
		if ( !p[0]->bExists ) bExists = FALSE;
		Alpha += m[0];
	}

	if ( !bExists ) return;

	Vect P = Vect(0,0,0);
	switch ( nPoints ) {

	case 6:
		P += m[5] * p[5]->p;
	case 5:
		P += m[4] * p[4]->p;
	case 4:
		P += m[3] * p[3]->p;
	case 3:
		P += m[2] * p[2]->p;
	case 2:
		P += m[1] * p[1]->p;
	case 1:
		P += m[0] * p[0]->p;
	}

	GeoPoint::p = ( 1/Alpha )*P;
}

bool GeoBary::bNeedPoint() {

	return TRUE;
}

void GeoBary::DoCheckDep() {

	bool bDestroy = FALSE;

	switch ( nPoints ) {

	case 6:
		if ( p[5]->bDestroying ) bDestroy = TRUE;
	case 5:
		if ( p[4]->bDestroying ) bDestroy = TRUE;
	case 4:
		if ( p[3]->bDestroying ) bDestroy = TRUE;
	case 3:
		if ( p[2]->bDestroying ) bDestroy = TRUE;
	case 2:
		if ( p[1]->bDestroying ) bDestroy = TRUE;
	case 1:
		if ( p[0]->bDestroying ) bDestroy = TRUE;
	}
	if ( bDestroy ) Destroy();
}

bool GeoBary::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

		obj->bMark = TRUE;
		p[ iPoints ] = (GeoPoint*)obj;
		iPoints++;
		if ( iPoints == nPoints ) {
			BuildDone();
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoBary::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoBary::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_BARY );
  ar << bVisible << Letter[0] << Letter[1] << Col << nPoints;
  switch ( nPoints ) {	
  case 6 :
    ar << p[5]->Index;
  case 5 :
    ar << p[4]->Index;
  case 4 :
    ar << p[3]->Index;
  case 3 :
    ar << p[2]->Index;
  case 2 :
    ar << p[1]->Index;
  case 1 :
    ar << p[0]->Index;
  }
  ar << m[0] << m[1] << m[2] << m[3] << m[4] << m[5];
}

void GeoBary::DoLinkFromIndex() {

	switch ( nPoints ) {
	case 6 :
		p[5] = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I[5] );
	case 5 :
		p[4] = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I[4] );
	case 4 :
		p[3] = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I[3] );
	case 3 :
		p[2] = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I[2] );
	case 2 :
		p[1] = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I[1] );
	case 1 :
		p[0] = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I[0] );
	}
}

void GeoBary::Properties() {

  Pln->StoreUndo();
  diaBary dia( Name, Letter, FALSE, nPoints, m );
  dia.run();
  for ( int i = 0 ; i < 6 ; i++ ) m[i] = dia.getWeight(i);
}

void GeoBary::SelectFromParents() {

	switch ( nPoints ) {

	case 6:
		if ( p[5]->bSelect ) bSelect = TRUE;
	case 5:
		if ( p[4]->bSelect ) bSelect = TRUE;
	case 4:
		if ( p[3]->bSelect ) bSelect = TRUE;
	case 3:
		if ( p[2]->bSelect ) bSelect = TRUE;
	case 2:
		if ( p[1]->bSelect ) bSelect = TRUE;
	case 1:
		if ( p[0]->bSelect ) bSelect = TRUE;
	}
	GeoBase::SelectFromParents();
}

///////////////////////////////////////////////////////////////////
// GeoCercleTangent -----------------------------------------------
// Cercle tangent  une droite ------------------------------------

GeoCercleTangent::GeoCercleTangent( Plan *pln ) : GeoCircle( pln, IDSN_CERCLE_TANGENT ) {

	bl = bc = FALSE;
}

GeoCercleTangent::GeoCercleTangent( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoCircle( pln, IDSN_CERCLE_TANGENT) {

	Index = index;
	ar >> Width >> Style >> bVisible >> Col >> Il >> Ic;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoCercleTangent::GeoCercleTangent( Plan *pln, GeoPoint *_c, GeoLine *_l ):GeoCircle( pln, IDSN_CERCLE_TANGENT) {

	l = _l; c = _c;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoCercleTangent::DoDefine() {

	if ( l ) l->Define();
	if ( c ) c->Define();
	if (( l->bExists )&&( c->bExists )) {

		Vect o = c->p;
		Vect p = l->o;
		Vect u = l->u;
		Vect op = p-o;
		double scaluop = u|op;

		r = sqrt( op.norm2() - scaluop*scaluop );
		GeoCircle::o = o;

		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoCercleTangent::DoCheckDep() {

	if (( l->bDestroying )||( c->bDestroying ))
		Destroy();
}

bool GeoCercleTangent::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpLine ) {

			l = (GeoLine*)obj;
			bl = TRUE;
			obj->bMark = TRUE;
		}
	if ( obj->Type & tpPoint ) {

			c = (GeoPoint*)obj;
			bc = TRUE;
			obj->bMark = TRUE;
		}
	if ( bl && bc ) {

		BuildDone();
		return TRUE;
	}
	return FALSE;
}

bool GeoCercleTangent::bNeedPoint() {

	return !bc;
}

bool GeoCercleTangent::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoCercleTangent::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_CERCLE_TANGENT );
  ar << bVisible << Col << l->Index << c->Index;
}

void GeoCercleTangent::DoLinkFromIndex() {

	l = (GeoLine*)Pln->Racine->GetBaseFromIndex( Il );
	c = (GeoPoint*)Pln->Racine->GetBaseFromIndex( Ic );
}

void GeoCercleTangent::SelectFromParents() {

	if ( c->bSelect ) bSelect = TRUE;
	GeoCircle::SelectFromParents();
}

///////////////////////////////////////////////////////////////////
// GeoProjLine ----------------------------------------------------
// Project d'un point sur une droite parallelement... ------------

GeoProjLine::GeoProjLine( Plan *pln ) : GeoPoint( pln, IDSN_PROJ_LINE ) {

	bl1 = bl2 = bc = FALSE;
}

GeoProjLine::GeoProjLine( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoPoint( pln, IDSN_PROJ_LINE ) {

	Index = index;
	ar >> bVisible >> Letter[0] >> Letter[1] >> Col >> Il1 >> Il2 >> Ic;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoProjLine::GeoProjLine( Plan *pln, GeoPoint *_c, GeoLine *_l1, GeoLine *_l2 ):GeoPoint( pln, IDSN_PROJ_LINE ) {

	l2 = _l2; l1 = _l1; c = _c;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoProjLine::DoDefine() {

	if ( l1 ) l1->Define();
	if ( l2 ) l2->Define();
	if ( c ) c->Define();
	if (( l1->bExists )&&( l2->bExists )&&( c->bExists )) {

		Vect o1 = c->p; Vect u1 = l1->u;
		Vect o2 = l2->o - o1; Vect u2 = l2->u;

		double t = (( u2.X[0] * o2.X[1] ) - ( u2.X[1] * o2.X[0] ));
		double den = (( u1.X[0] * u2.X[1] ) - ( u1.X[1] * u2.X[0] ));
		if ( !den ) {
			bExists = FALSE;
			return;
		}

		t = t/den;
		p = o1 - t*u1;

		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoProjLine::DoCheckDep() {

	if (( l1->bDestroying )||( l2->bDestroying )||( c->bDestroying ))
		Destroy();
}

bool GeoProjLine::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpLine ) {

		obj->bMark = TRUE;
		if ( !bl1 ) {
			l1 = (GeoLine*)obj;
			bl1 = TRUE;
		}
		else if ( !bl2 ) {
			l2 = (GeoLine*)obj;
			bl2 = TRUE;
		}
	}
	if ( obj->Type & tpPoint ) {

			c = (GeoPoint*)obj;
			bc = TRUE;
			obj->bMark = TRUE;
		}
	if ( bl1 && bl2 && bc ) {

		BuildDone();
		return TRUE;
	}
	return FALSE;
}

bool GeoProjLine::bNeedPoint() {

	return !bc;
}

bool GeoProjLine::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoProjLine::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_PROJ_LINE );
  ar << bVisible << Letter[0] << Letter[1] << Col << l1->Index << l2->Index << c->Index;
}

void GeoProjLine::DoLinkFromIndex() {

	l1 = (GeoLine*)Pln->Racine->GetBaseFromIndex( Il1 );
	l2 = (GeoLine*)Pln->Racine->GetBaseFromIndex( Il2 );
	c = (GeoPoint*)Pln->Racine->GetBaseFromIndex( Ic );
}

/////////////////////////////////////////////////////////////////////////////////////
// GeoNorm -- 

GeoNorm::GeoNorm( Plan* pln ):GeoComment( pln, IDSN_NORM ) {

	nPoints = 0;
}

GeoNorm::GeoNorm( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ):GeoComment( pln, IDSN_NORM ) {

	nPoints = 0;
	Index = index;
	ar >> bVisible >> Col >> I1 >> I2 >> p1.X[0] >> p1.X[1];
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoNorm::GeoNorm( Plan *pln, GeoPoint* pp1, GeoPoint* pp2 ):GeoComment( pln, IDSN_NORM ) {

	_p1 = pp1; _p2 = pp2;
	p1 = Pln->GetFreeRegion();
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoNorm::DoPaint() {

 	Vect p10 = Pln->GetDcCoords( _p1->p );
 	Vect p20 = Pln->GetDcCoords( _p2->p );
	
	String = "(";
	char c1[3]; 
	char c2[3];
	c1[0] = _p1->Letter[0];  c1[1] = _p1->Letter[1]; c1[2] = 0;	
	c2[0] = _p2->Letter[0];	 c2[1] = _p2->Letter[1]; c2[2] = 0;
	if ( c1[0] ) String += c1; else String += '?';
	String += ',';
	if ( c2[0] ) String += c2; else String += '?';
	String += ") = ";
	char tmp[64];
	double Unite = ( Pln->Unite )?( Pln->Unite->Norme ):1;
	sprintf( tmp, "%g", Norme / Unite );
	String += tmp;

	GeoComment::DoPaint();
}

void GeoNorm::DoDefine() {

	if ( _p1 ) _p1->Define();
	if ( _p2 ) _p2->Define();
	if (( _p1->bExists )&&( _p2->bExists )) {

		Norme = ( _p2->p - _p1->p ).norm();
		bExists = TRUE;
	}
	else bExists = FALSE;	
}

void GeoNorm::DoCheckDep() {

	if (( _p1->bDestroying )||( _p2->bDestroying )) Destroy();
}

bool GeoNorm::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

		if ( nPoints == 0 ) {
			_p1 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 1 )&&( _p1 != obj )) {
			_p2 = (GeoPoint*)obj;
			obj->bMark = TRUE;
			p1 = Pln->GetFreeRegion();
			BuildDone();
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoNorm::bNeedPoint() {

	return TRUE;
}

bool GeoNorm::AddPoint( Vect& scrnpoint ) {

	return FALSE;
}

void GeoNorm::DoStore( CMyArchive& ar ) {

  ar.setType( IDM_NORM );
  ar << bVisible << Col << _p1->Index << _p2->Index << p1.X[0] << p1.X[1];
}

void GeoNorm::DoLinkFromIndex() {

	_p1 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I1 );
	_p2 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
}

void GeoNorm::Properties() {

  Pln->StoreUndo();
  bSliding = FALSE;
  bool bBeforeUnit = ( Pln->Unite == this );
  diaNorm dia( Name, Letter, bBeforeUnit );
  if ( dia.run() ) Pln->Unite = this;
  else if ( bBeforeUnit ) Pln->Unite = NULL;
}

//////////////////////////////////////////////////////////////////
// GeoCercle3 ----------------------------------------------------
// Cercle passant par 3 points -----------------------------------

GeoCercle3::GeoCercle3( Plan *pln ) : GeoCircle( pln, IDSN_CERCLE3 ) {

	nPoints = 0;
}

GeoCercle3::GeoCercle3( Plan *pln, int index, GeoBase* racine, CMyArchive& ar ) : GeoCircle( pln, IDSN_CERCLE3 ) {

	Index = index;
	ar >> Width >> Style >> bVisible >> Col >> I1 >> I2 >> I3;
	Suivant = Pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	Status = TRUE;
}

GeoCercle3::GeoCercle3( Plan *pln, GeoPoint *_p1, GeoPoint *_p2, GeoPoint *_p3 ):GeoCircle( pln, IDSN_CERCLE3 ) {

	p1 = _p1; p2 = _p2; p3 = _p3;
	Suivant = pln->Racine;
	if ( Suivant ) Suivant->Precedent = this;
	pln->Racine = this;
	Status = TRUE;
}

void GeoCercle3::DoDefine() {

	if ( p1 ) p1->Define();
	if ( p2 ) p2->Define();
	if ( p3 ) p3->Define();
	if (( p1->bExists )&&( p2->bExists )&&( p3->bExists )) {

		Vect a = p1->p;
		Vect b = p2->p;
		Vect c = p3->p;

		Vect mc = 0.5*( a+b );
		Vect mb = 0.5*( a+c );

		Vect uc = !(( a-b )*Vect( 0,0,1 ));
		Vect ub = !(( a-c )*Vect( 0,0,1 ));

		mc = mc-mb;

		double t = (( uc.X[0] * mc.X[1] ) - ( uc.X[1] * mc.X[0] ));
		double den = (( ub.X[0] * uc.X[1] ) - ( ub.X[1] * uc.X[0] ));
		if ( fabs( den ) < 1e-10 ) {
			bExists = FALSE;
			return;
		}

		t = t/den;
		o = mb + (-t*ub);
		
		r = ( a - o ).norm();

		bExists = TRUE;
	}
	else bExists = FALSE;
}

void GeoCercle3::DoCheckDep() {

	if (( p1->bDestroying )||( p2->bDestroying )||( p3->bDestroying ))
		Destroy();
}

bool GeoCercle3::AddObject( GeoBase* obj ) {

	if ( obj->Type & tpPoint ) {

		if ( nPoints == 0 ) {
			p1 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 1 )&&( p1 != obj )) {
			p2 = (GeoPoint*)obj;
			nPoints++;
			obj->bMark = TRUE;
		}
		if (( nPoints == 2 )&&( p2 != obj )&&( p1 != obj )) {
			p3 = (GeoPoint*)obj;
			BuildDone();
			return TRUE;
		}
	}
	return FALSE;
}

bool GeoCercle3::bNeedPoint() {

	return TRUE;
}

bool GeoCercle3::AddPoint( Vect& scrnpoint ) { return FALSE; }

void GeoCercle3::DoStore( CMyArchive& ar ) {
  
  ar.setType( IDM_CERCLE3 );
  ar << Width << Style << bVisible << Col << p1->Index << p2->Index << p3->Index;
}

void GeoCercle3::DoLinkFromIndex() {

	p1 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I1 );
	p2 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I2 );
	p3 = (GeoPoint*)Pln->Racine->GetBaseFromIndex( I3 );
}

void GeoCercle3::SelectFromParents() {

	GeoCircle::SelectFromParents();
}

