/*  Copyright 2000-2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "id.h"
#include "MyArchive.h"
#include "Plane.h"
#include "GeoBase.h"
#include "GeoComment.h"
#include "GeoVector.h"
#include "euclideView.h"
#include "euclideDocument.h"

Plan::Plan( euclideDocument* _doc, double szx, double szy ) {

  doc = _doc;
  MouseMarge = DEFAULT_MOUSEMARGE;
  rAngle = DEFAULT_RANGLE;
  bDegrees = TRUE;
  
  LoadOptions();
  
  NewObj = NULL;
  Unite = NULL;
  yFreeRegion = -0.95;
  bErrorStructure = FALSE;
  UndoBuffer = new CUndoBuffer( DEFAULT_N_UNDO );
  nCurs = CURS_NORMAL;
  TargetObject = NULL;
  szX = szx; szY = szy;
  xMin = -1; xMax = 1; yMin = -1; yMax = 1;
  strX = szx; strY = szy;
  Status = FALSE;
  Racine = NULL;
  bDestroying = FALSE;
  bModified = FALSE;
}

Plan::~Plan() { 

  if (Racine) Racine->DeleteAll();
  delete UndoBuffer;
}

Vect Plan::GetFreeRegion() {

  yFreeRegion += 0.05;
  if ( yFreeRegion > 0.0 ) yFreeRegion = -0.95;
  return Vect( -0.9, yFreeRegion, 0 );
}

void Plan::StoreUndo() {

  CMyArchive* buf = UndoBuffer->GetNextBuffer();
  Store( *buf );
  buf->CloseEvaluating();
  Store( *buf );
  buf->CloseStoring();
  bModified = TRUE;
}

bool Plan::Undo() {

  bModified = TRUE;
  if ( UndoBuffer->OnTheTop() ) {
    StoreUndo();
    UndoBuffer->GetUndoBuffer();
  }
  CMyArchive* buf = UndoBuffer->GetUndoBuffer();
  if ( buf ) {
    
    Load( *buf );
    InvalidateAllOfThem();
  } 
  else Command( IDM_NEW );
  return ( buf != 0 );
}

void Plan::Redo() {

  bModified = TRUE;
  CMyArchive* buf = UndoBuffer->GetRedoBuffer();
  if ( buf ) {
    
    Load( *buf );
    InvalidateAllOfThem();
  }
}

bool Plan::GetUndoState() {

	return UndoBuffer->GetUndoState();
}

bool Plan::GetRedoState() {

	return UndoBuffer->GetRedoState();
}

void Plan::UnSelect() {

  if (( !ctrlStatus )&&( Racine )) {
    Racine->UnSelect( TRUE );
    InvalidateAllOfThem();
  }
}

void Plan::SelectFromZone( Vect p1, Vect p2 ) {
  
  if ( Racine ) {
    Racine->SelectFromZone( p1, p2 );
    Racine->SelectFromParents();
    InvalidateAllOfThem();
  }
}

void Plan::LoadOptions() {
  
  // 	UINT pBytes = MAX_COULEUR*sizeof(COLORREF);
  // 	LPBYTE Ptr;
  
// 	// Les couleurs --- 

// 	if ( AfxGetApp()->GetProfileBinary( "Options", "Colors", &Ptr, &pBytes ) )
// 		memcpy( TabCols, Ptr, pBytes );
}

void Plan::SaveOptions() {

// 	UINT pBytes = MAX_COULEUR*sizeof(COLORREF);

// 	// Les couleurs ---

// 	AfxGetApp()->WriteProfileBinary( "Options", "Colors", (LPBYTE)TabCols, pBytes );
}

void Plan::SetScreenFrame( long strx, long stry ) {

  strX = strx; strY = strx;
  
  doc->aVoid();
  if ( Racine ) Racine->Paint();
}

void Plan::Zoom( bool in ) {
  
  if ( in ) {
    
    xMin/=1.5; xMax/=1.5;
    yMin/=1.5; yMax/=1.5;
  }
  else {
    
    xMin*=1.5; xMax*=1.5;
    yMin*=1.5; yMax*=1.5;
  }
}

Vect Plan::GetScreenCoords( Vect mathpoint ) {
  
  return Vect( 
	      ( ( mathpoint.X[0] - xMin ) / ( xMax - xMin ) ) * strX,
	      ( ( mathpoint.X[1] - yMin ) / ( yMax - yMin ) ) * strY,
	      0 );
}

Vect Plan::GetDcCoords( Vect mathpoint ) { // long long ago, "Screen" and "Dc" were different ...
  
  return Vect( 
	      ( ( mathpoint.X[0] - xMin ) / ( xMax - xMin ) ) * strX,
	      ( ( mathpoint.X[1] - yMin ) / ( yMax - yMin ) ) * strY,
	      0 );
}

Vect Plan::GetMathCoords( Vect scrnpoint ) {
  
  return Vect(
	      xMin + ( scrnpoint.X[0] / strX )*( xMax - xMin ),
	      yMin + ( scrnpoint.X[1] / strY )*( yMax - yMin ),
	      0 );
}

void Plan::selectAll() {
  
  if ( Racine ) Racine->SelectAll();
  InvalidateAllOfThem();
}

int Plan::MouseEvent( TEvent evt, double x, double y ) {
  
  Vect scrnpoint( x, y, 0 );
  
  if ( Status ) {
    
    if (( evt == evLMouse )&&(NewObj->AddPoint( scrnpoint ) )) {
      // Always send the point before the object: for FixPointLimited
      
      Status = FALSE; Racine->UnMark(); lpstr.empty(); nCurs = CURS_NORMAL;
      InvalidateAllOfThem();
      return 1;
    }
    
    GeoBase* obj = Racine ? Racine->Event( evt, scrnpoint ) : NULL;
    
    if (( evt == evLMouse )&&( !obj )&&( NewObj->bNeedPoint() )) {
      
      GeoFixPoint* Pt = new GeoFixPoint( this, scrnpoint.X[0], scrnpoint.X[1] );
      if ( NewObj->AddObject( Pt ) ) {
	
	Status = FALSE; Racine->UnMark(); lpstr.empty(); nCurs = CURS_NORMAL;
	InvalidateAllOfThem();
	return 1;
      }
      InvalidateAllOfThem();
      return 1;
    }
    
    if (( evt == evLMouse )&&( obj )) {
      
      if ( NewObj->AddObject( obj ) ) {
	
	Status = FALSE; Racine->UnMark(); lpstr.empty(); nCurs = CURS_NORMAL;
	InvalidateAllOfThem();
	return 1;
      }
      InvalidateAllOfThem();
      return 1;
    }
    if (( evt == evRMouse )&&( obj )) {
      TargetObject = obj;
      return 2;
    }
    return 0;
  }
  else 
    if ( Racine ) {	
      
      GeoBase* obj = Racine->Event( evt, scrnpoint );
      if (( evt == evRMouse )&&( obj )) {
	TargetObject = obj;
	return 2;
      }
      if (( evt == evLMouseDblClk )&&( obj )) {
	obj->Properties();
	InvalidateAllOfThem();
	return 0;
      }
      if ( evt == evLMouse ) {
	if ( ctrlStatus ) {
	  if ( obj ) obj->ChgSelect();
	}
	else { 
	  Racine->UnSelect( TRUE );
	  if ( obj ) obj->Select();
	}
	InvalidateAllOfThem();
      }
      if ( obj ) {
	return 1;
      }
      return 0;
    }
  return 0;
}

void Plan::NewObject( int idStatus ) {

  nCurs = CURS_SELECT;
  if ( Status ) { 
    delete NewObj; 
    if ( Racine ) Racine->UnMark(); 
  }
  Status = idStatus;
  if ( Racine ) Racine->UnSelect( TRUE );
}

bool Plan::Command( int id ) {

  switch( id ) {
    
  case IDM_NEW:
    if ( Racine ) {
      //      StoreUndo();
      Racine->DeleteAll();
      Racine = NULL;
    }
    if ( Status ) delete NewObj;
    Status = FALSE;
    InvalidateAllOfThem();
    break;
  case IDM_FIXPOINT:
    lpstr.loadString( IDS_FIXPOINT );
    NewObject(id);
    NewObj = new GeoFixPoint( this );
    nCurs = CURS_FIXPOINT;
    break;
  case IDM_MIDPOINT:
    lpstr.loadString( IDS_MIDPOINT );
    NewObject(id);
    NewObj = new GeoMidPoint( this );
    break;
  case IDM_VECT:
    lpstr .loadString( IDS_VECT );
    NewObject(id);
    NewObj = new GeoVect( this );
    break;
  case IDM_CARRE:
    lpstr .loadString( IDS_CARRE );
    NewObject(id);
    NewObj = new GeoCarre( this );
    break;
  case IDM_LINEAB:
    lpstr .loadString( IDS_LINEAB );
    NewObject(id);
    NewObj = new GeoLineAB( this );
    break;
  case IDM_ORTHOGONAL:
    lpstr .loadString( IDS_ORTHOGONAL );
    NewObject(id);
    NewObj = new GeoOrthogonal( this );
    break;
  case IDM_PARALLEL:
    lpstr .loadString( IDS_PARALLEL );
    NewObject(id);
    NewObj = new GeoParallel( this );
    break;
  case IDM_POINTINTERSECT:
    lpstr.loadString( IDS_POINTINTERSECT );
    NewObject(id);
    NewObj = new GeoPointIntersect( this );
    break;
  case IDM_CIRCLEAB:
    lpstr.loadString( IDS_CIRCLEAB );
    NewObject(id);
    NewObj = new GeoCircleAB( this );
    break;
  case IDM_SEGMENT:
    lpstr.loadString( IDS_SEGMENT );
    NewObject(id);
    NewObj = new GeoLineAB( this );
    ((GeoLineAB*)NewObj)->IsSegment();
    break;
  case IDM_ANGLE:
    lpstr.loadString( IDS_ANGLE );
    NewObject(id);
    NewObj = new GeoAngle( this );
    break;
  case IDM_ANGLE_GEO:
    lpstr.loadString( IDS_ANGLE_GEO );
    NewObject(id);
    NewObj = new GeoAngle( this );
    ((GeoAngle*)NewObj)->bOriented = FALSE;
    break;
  case IDM_POINT_ROTATED:
    lpstr.loadString( IDS_POINT_ROTATED );
    NewObject(id);
    NewObj = new GeoPointRotated( this );
    break;
  case IDM_FIXPOINT_LIMITED:
    lpstr.loadString( IDS_FIXPOINT_LIMITED );
    NewObject(id);
    NewObj = new GeoFixPoint( this, TRUE );
    break;
  case IDM_COMMENT:
    lpstr.loadString( IDS_COMMENT );
    NewObject(id);
    NewObj = new GeoComment( this, IDSN_COMMENT );
    break;
  case IDM_PROJ_CIRCLE:
    lpstr.loadString( IDS_PROJ_CIRCLE );
    NewObject(id);
    NewObj = new GeoProjCercle( this );
    break;
  case IDM_BISSECTRICE:
    lpstr.loadString( IDS_BISSECTRICE );
    NewObject(id);
    NewObj = new GeoBissectrice( this );
    break;
  case IDM_INTER_CERCLE_DROITE:
    lpstr.loadString( IDS_INTER_CERCLE_DROITE );
    NewObject(id);
    NewObj = new GeoInterCercleDroite( this );
    break;
  case IDM_INTER_CERCLE_CERCLE:
    lpstr.loadString( IDS_INTER_CERCLE_CERCLE );
    NewObject(id);
    NewObj = new GeoInterCercleCercle( this );
    break;
  case IDM_BARY:
    lpstr.loadString( IDS_BARY );
    NewObject(id);
    NewObj = new GeoBary( this );
    break;
  case IDM_CERCLE_TANGENT:
    lpstr.loadString( IDS_CERCLE_TANGENT );
    NewObject(id);
    NewObj = new GeoCercleTangent( this );
    break;
  case IDM_PROJ_LINE:
    lpstr.loadString( IDS_PROJ_LINE );
    NewObject(id);
    NewObj = new GeoProjLine( this );
    break;
  case IDM_NORM:
    lpstr.loadString( IDS_NORM );
    NewObject(id);
    NewObj = new GeoNorm( this );
    break;
  case IDM_PROJ_ORTHOGONAL:
    lpstr.loadString( IDS_PROJ_ORTHOGONAL );
    NewObject(id);
    NewObj = new GeoProjOrthogonal( this );
    break;
  case IDM_CERCLE3:
    lpstr.loadString( IDS_CERCLE3 );
    NewObject(id);
    NewObj = new GeoCercle3( this );
    break;
  case IDM_SYMETRIQUE:
    lpstr.loadString( IDS_SYMETRIQUE );
    NewObject(id);
    NewObj = new GeoProjOrthogonal( this, TRUE );
    break;
  case IDM_VECTORAB:
    lpstr.loadString( IDS_VECTORAB );
    NewObject(id);
    NewObj = new GeoVectorAB( this );
    break;		
  case IDM_VECTORSUM:
    lpstr.loadString( IDS_VECTORSUM );
    NewObject(id);
    NewObj = new GeoVectorSum( this, FALSE );
    break;		
  case IDM_VECTORDIFF:	  
    lpstr.loadString( IDS_VECTORDIFF );
    NewObject(id);
    NewObj = new GeoVectorSum( this, TRUE );
    break;		
  case IDM_TRANSLATEDPOINT:	  
    lpstr.loadString( IDS_TRANSLATEDPOINT );
    NewObject(id);
    NewObj = new GeoTranslatedPoint( this );
    break;		
  case IDM_SCALAIREPROD:
    lpstr.loadString( IDS_SCALAIREPROD );
    NewObject(id);
    NewObj = new GeoScalaireProd( this );
    break;		
	case IDM_DESTROY:
		if ( TargetObject ) {
			StoreUndo();
			TargetObject->Destroy();
		}
		InvalidateAllOfThem();
		TargetObject = NULL;
		return TRUE;
	case IDM_PROPERTIES:
		if ( TargetObject ) {
			StoreUndo();
			TargetObject->Properties();
		}
		break;
	case IDM_DELETE:
		if ( Racine ) {
			StoreUndo();
			Racine->DeleteSelected();
		}
		break;
	case IDM_SELECTALL:
	  if ( Racine ) Racine->SelectAll();
	  break;
	case IDM_INVISIBLE:
	  if ( Racine ) {
	    StoreUndo();
	    Racine->InvisibleSelected();
	  }
	  break;
	case IDM_HIDE:
	  if ( Racine ) {
	    StoreUndo();
	    Racine->HideSelected();
	  }
	  break;
	case IDM_RELAX: {

	  if ( Status ) {
	    delete NewObj;
	    Status = FALSE;
	  }
	  if ( Racine ) Racine->UnMark();
	  nCurs = CURS_NORMAL;
	}
	case IDM_LEFT:
		if ( Racine ) {
		
			GeoBase* obj = Racine->GetSelected();
			if ( obj ) obj->Left();
		}
		break;
	case IDM_RIGHT:
		if ( Racine ) {
		
			GeoBase* obj = Racine->GetSelected();
			if ( obj ) obj->Right();
		}
		break;
	case IDM_UP:
		if ( Racine ) {
		
			GeoBase* obj = Racine->GetSelected();
			if ( obj ) obj->Up();
		}
		break;
	case IDM_DOWN:
		if ( Racine ) {
		
			GeoBase* obj = Racine->GetSelected();
			if ( obj ) obj->Down();
		}
		break;
	case IDM_COPY:
		Copy();
		break;
	}
  mlString mlCreation( IDS_CREATIONOF );
  view->setMessageText( (string)mlCreation + (string)lpstr );
  InvalidateAllOfThem();
  TargetObject = NULL;
  return TRUE;
}

void Plan::InvalidateAllOfThem() {
	// Objects and painting are all to be redefined 

	if (( bDestroying )&&( Racine )) {
		bDestroying = FALSE;
		Racine->BurnTheDeads();	  
	}

	if ( Racine ) {
		Racine->InvalidateAllOfThem();
		Racine->DefineAll();		    
	}

	doc->aVoid();
	rAngle = (int)GetScreenCoords( Vect( -0.95,0,0 ) ).X[0];
	if ( Racine ) Racine->Paint();
	doc->invalidate();
}

void Plan::Store( CMyArchive& ar ) {

  if ( Racine ) {
    Racine->SetIndex();
    if ( Unite ) ar << (int)Unite->Index; else ar << (int)-1;
    Racine->Store( ar );
  } 
  else ar << (int)0;
  
  ar << IDM_DESTROY;		// End marker
}

void Plan::Load( CMyArchive& ar ) {

  Command( IDM_NEW );
  //	if ( ar.IsFile() ) StoreUndo();
  
  int IndexUnite; 
  ar >> IndexUnite;
  
  int Type = 0; 
  int Index = 0;
  while( ar.IsLoading() && ( Type != IDM_DESTROY )) {
    
    Type = ar.getType();
    switch ( Type ) {
      
    case IDM_FIXPOINT: Racine = new GeoFixPoint( this, Index++, Racine, ar );
      break;
    case IDM_POINTINTERSECT: Racine = new GeoPointIntersect( this, Index++, Racine, ar );
      break;
    case IDM_MIDPOINT: Racine = new GeoMidPoint( this, Index++, Racine, ar );
      break;
    case IDM_VECT: Racine = new GeoVect( this, Index++, Racine, ar );
      break;
    case IDM_CARRE: Racine = new GeoCarre( this, Index++, Racine, ar );
      break;
    case IDM_LINEAB: Racine = new GeoLineAB( this, Index++, Racine, ar );
      break;
    case IDM_ORTHOGONAL: Racine = new GeoOrthogonal( this, Index++, Racine, ar );
      break;
    case IDM_PARALLEL: Racine = new GeoParallel( this, Index++, Racine, ar );
      break;
    case IDM_CIRCLEAB: Racine = new GeoCircleAB( this, Index++, Racine, ar );
      break;
    case IDM_ANGLE: Racine = new GeoAngle( this, Index++, Racine, ar );
      break;
    case IDM_POINT_ROTATED: Racine = new GeoPointRotated( this, Index++, Racine, ar );
      break;
    case IDM_COMMENT: Racine = new GeoComment( this, Index++, Racine, ar );
      break;
    case IDM_PROJ_CIRCLE: Racine = new GeoProjCercle( this, Index++, Racine, ar );
      break;
    case IDM_BISSECTRICE: Racine = new GeoBissectrice( this, Index++, Racine, ar );
      break;
    case IDM_INTER_CERCLE_DROITE: Racine = new GeoInterCercleDroite( this, Index++, Racine, ar );
      break;
    case IDM_INTER_CERCLE_CERCLE: Racine = new GeoInterCercleCercle( this, Index++, Racine, ar );
      break;
    case IDM_BARY: Racine = new GeoBary( this, Index++, Racine, ar );
      break;
    case IDM_CERCLE_TANGENT: Racine = new GeoCercleTangent( this, Index++, Racine, ar );
      break;
    case IDM_PROJ_LINE: Racine = new GeoProjLine( this, Index++, Racine, ar );
      break;
    case IDM_NORM: Racine = new GeoNorm( this, Index++, Racine, ar );
      break;
    case IDM_PROJ_ORTHOGONAL: Racine = new GeoProjOrthogonal( this, Index++, Racine, ar );
      break;
    case IDM_CERCLE3: Racine = new GeoCercle3( this, Index++, Racine, ar );
      break;
    case IDM_SYMETRIQUE: Racine = new GeoProjOrthogonal( this, Index++, Racine, ar, TRUE );
      break;
    case IDM_VECTORAB: Racine = new GeoVectorAB( this, Index++, Racine, ar );
      break;
    case IDM_VECTORSUM: Racine = new GeoVectorSum( this, Index++, Racine, ar );
      break;
    case IDM_TRANSLATEDPOINT: Racine = new GeoTranslatedPoint( this, Index++, Racine, ar );
      break;
    case IDM_SCALAIREPROD: Racine = new GeoScalaireProd( this, Index++, Racine, ar );
      break;
    case IDM_DESTROY:
      break;
    default :
      //			AfxMessageBox( IDS_ERROR_VERSION );
      Undo();
      return;
    }
  }
  
  if (( Racine )&&( IndexUnite != -1 )) Unite = (GeoNorm*)Racine->GetBaseFromIndex( IndexUnite );
  else Unite = NULL;
  if ( Racine ) Racine->LinkFromIndex();
  if ( bErrorStructure ) {
    //		AfxMessageBox( IDS_ERROR_STRUCTURE );
    //		if ( ar.IsFile() ) Undo();
    Command( IDM_NEW );
  }
  InvalidateAllOfThem();
}

void Plan::DispatchNewLetter( char letter ) {

  if ( Racine ) Racine->DispatchNewLetter( letter );
  InvalidateAllOfThem();
}

void Plan::ctSelected() {

	nSelected = 0;
	if ( Racine ) Racine->ctSelected();
}

void Plan::SelectToTarget() {

	if ( Racine ) TargetObject = Racine->GetSelected();
	else TargetObject = NULL;
}

void Plan::Copy() {

// 	DiaCopyRes dia;
// 	if ( dia.DoModal() != IDOK ) return;

// 	if ( Racine ) Racine->UnSelect( TRUE );
	
// 	double OldStrX = strX;
// 	strX = strY = dia.GetNRes();

// 	HDC hScrDc = GetDC( NULL );
// 	HDC hDc = CreateCompatibleDC( hScrDc );
// 	HBITMAP hBm = CreateCompatibleBitmap( hScrDc, (int)strX, (int)strY );
// 	HBITMAP hBmOld = (HBITMAP)SelectObject( hDc, hBm );
// 	ReleaseDC( NULL, hScrDc );
// 	InvalidateAllOfThem( hDc );
// 	SelectObject( hDc, hBmOld );
// 	DeleteDC( hDc );

// 	CEuclideMfcApp* App = (CEuclideMfcApp*)AfxGetApp();

// 	if ( OpenClipboard( App->m_pMainWnd->m_hWnd ) ) {

// 		EmptyClipboard();
// 		SetClipboardData( CF_BITMAP, hBm );
// 		CloseClipboard();
// 	}

// 	strX = strY = OldStrX;
}
