/*  Copyright 2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _VECT_H_ 
#define _VECT_H_

#include <cmath>
#include <fstream>
#include <iostream>
#include <cstring>
using namespace std;

#ifndef M_PI
#define M_PI 3.14159265
#endif

#define INV_INFINI 1e-30

inline double sqr( double x ) {

	return x*x;
}

inline double _acos( double x ) {

	if ( x > 1 ) return 0;
	if ( x < -1 ) return M_PI;
	return acos(x);
}

inline double _asin( double x ) {

	if ( x > 1 ) return M_PI/2;
	if ( x < -1 ) return -M_PI/2;
	return asin(x);
}

class Vect {
public:
 double X[3];

 Vect () { X[0] = 0; X[1] = 0; X[2] = 0; }
 Vect ( double u, double v, double w );
 bool nul();
 Vect operator + ( Vect a );
 double norm ();
 double norm2 ();
 Vect operator - ( Vect a );
 Vect& operator += ( Vect a );
 Vect& operator -= ( Vect a );
 Vect operator * ( double s );
 Vect operator * ( Vect v );  // Produit vectoriel
 double operator | ( Vect a );
 Vect& operator !();
 Vect operator -();
 double angle();
 inline double x( int n ) const { return X[n]; } 
};

inline Vect operator * ( double s, Vect a ) { return a*s; }

inline ifstream&  operator >> ( ifstream& is, Vect& v ) {

  is >> v.X[0];
  is >> v.X[1];
  is >> v.X[2];
  return is;
}

inline ofstream&  operator << ( ofstream& os, Vect v ) {

  os << v.X[0];
  os << v.X[1];
  os << v.X[2];
  return os;
}

inline bool Vect::nul() {

	return ( !X[0]&&!X[1]&&!X[2] );
}

inline Vect Vect::operator -() {

	return Vect ( -X[0], -X[1], -X[2] );
}

inline Vect Vect::operator * ( double s ) {

	return ( Vect ( s*X[0], s*X[1], s*X[2] ) );
}

inline Vect Vect::operator * ( Vect v ) {

	return   Vect ( X[1]*v.X[2] - X[2]*v.X[1],
					X[2]*v.X[0] - X[0]*v.X[2],
					X[0]*v.X[1] - X[1]*v.X[0] );
}

inline Vect::Vect ( double u, double v, double w ) {

	X[0]=u; X[1]=v; X[2]=w;
}

inline double Vect::operator | ( Vect a ) {

	return ( X[0]*a.X[0] + X[1]*a.X[1] + X[2]*a.X[2] );
}

inline Vect& Vect::operator !() {

	double Norme = sqrt ( sqr(X[0]) + sqr(X[1]) + sqr(X[2]) );
	if ( !Norme ) {
		X[0] = X[1] = X[2]= 0;
		return *this;
	}

	double invNorm = 1 / Norme;
	X[0] *= invNorm;
	X[1] *= invNorm;
	X[2] *= invNorm;
	return *this;
}

inline Vect Vect::operator + ( Vect a ) {

 return Vect ( X[0]+a.X[0], X[1]+a.X[1], X[2]+a.X[2] );
}

inline double Vect::norm () {

  double Norme = sqrt ( sqr(X[0]) + sqr(X[1]) + sqr(X[2]) );
  if ( !Norme ) return INV_INFINI;
  return Norme;
}

inline double Vect::norm2 () {

  double Norme = sqr(X[0]) + sqr(X[1]) + sqr(X[2]);
  if ( !Norme ) return INV_INFINI;
  return Norme;
}

inline Vect Vect::operator - ( Vect a ) {

 return Vect ( X[0]-a.X[0], X[1]-a.X[1], X[2]-a.X[2] );
}

inline Vect& Vect::operator += ( Vect a ) {

	X[0] += a.X[0]; 
	X[1] += a.X[1]; 
	X[2] += a.X[2]; 
	return *this;
}

inline Vect& Vect::operator -= ( Vect a ) {

	X[0] -= a.X[0]; 
	X[1] -= a.X[1]; 
	X[2] -= a.X[2]; 
	return *this;	
}

inline double Vect::angle() {

  return (X[0]<0)?(atan( X[1]/X[0] )+M_PI):(atan( X[1]/X[0] ));
}

#endif
