/*  Copyright 2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "diaBase.h"
#include "mlstring.h"
#include "id.h"
#include <time.h>
#include <iostream>
using namespace std;

// diaBase : the root class for all dialogs -----------------------------------------

void dia_rt_func (GtkWidget *widget, gint arg1, gpointer user_data ) {

  ( (diaBase*)user_data )->rt_static( arg1 );
}

void diaBase::rt_static( gint arg1 ) { 

  if ( arg1 == GTK_RESPONSE_ACCEPT ) rt();
}

diaBase::diaBase( const string& _name ) {

  dialog = gtk_dialog_new_with_buttons (_name.c_str(),
					NULL,
					(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
  					GTK_STOCK_OK,
					GTK_RESPONSE_ACCEPT,
					GTK_STOCK_CANCEL,
					GTK_RESPONSE_REJECT, NULL);

  g_signal_connect (GTK_OBJECT (dialog),
		    "response",
		    G_CALLBACK (dia_rt_func),
		    (gpointer)this);
}

diaBase::~diaBase() {

  gtk_widget_destroy(dialog);
}

void diaBase::doRun() {

  gtk_dialog_run( GTK_DIALOG( dialog ) );
}

void diaBase::rt() {}

// errBase : root class for all error message popups ---------------------------------

errBase::errBase( int id ) {

  res = FALSE;
  dialog = NULL;
  str = mlString( id );
}

errBase::errBase( const string& message ) {
 
  res = FALSE;
  dialog = NULL;
  str = message;
}

void dia_errBase_rt_func (GtkWidget *widget, gint arg1, gpointer user_data ) {

  ( (errBase*)user_data )->rt_static( arg1 );
}

bool errBase::run( bool bAsk ) {

  if ( bAsk ) 
    dialog = gtk_message_dialog_new (NULL,
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_MESSAGE_QUESTION,
				     GTK_BUTTONS_YES_NO,
				     "%s", str.c_str());
  else 
    dialog = gtk_message_dialog_new (NULL,
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_MESSAGE_ERROR,
				     GTK_BUTTONS_OK,
				     "%s", str.c_str());
  g_signal_connect (GTK_OBJECT (dialog),
		    "response",
		    G_CALLBACK (dia_errBase_rt_func),
		    (gpointer)this);
  gtk_dialog_run (GTK_DIALOG (dialog));
  return res;
}

void errBase::rt_static( gint arg1 ) { 

  res = ( arg1 == GTK_RESPONSE_YES );
}

errBase::~errBase() {

  if ( dialog ) gtk_widget_destroy (dialog);
}

// diaComment : dialog class for comment areas ------------------------------

diaComment::diaComment( const string& _name, const string& _comment ) : diaBase( _name ) {

  comment = _comment;
  
  mlString strFrame( IDS_PROPERTIES_COMMENT );
  entryFrame = gtk_frame_new( ((string)strFrame).c_str() );
  
  entry = gtk_entry_new_with_max_length( 128 );
  gtk_entry_set_text( GTK_ENTRY(entry), comment.c_str() );
  gtk_container_add( GTK_CONTAINER( entryFrame ), entry );
  gtk_box_pack_start( GTK_BOX( GTK_DIALOG(dialog)->vbox ), entryFrame, TRUE, FALSE, 2 );

  gtk_widget_show_all( entryFrame );
}

string diaComment::run() {

  doRun();
  return comment;
}

void diaComment::rt() {

  comment = gtk_entry_get_text( GTK_ENTRY(entry) );
}

// diaObject : dialog class for basic object ------------------------------------------

diaObject::diaObject( const string& _name, char* _letter ) : diaBase( _name ) {

  letter = _letter;
  
  mlString strFrame( IDS_PROPERTIES_ENTRY );
  entryFrame = gtk_frame_new( ((string)strFrame).c_str() );
  
  entry = gtk_entry_new_with_max_length( 2 );
  gtk_entry_set_text( GTK_ENTRY(entry), letter );
  gtk_container_add( GTK_CONTAINER( entryFrame ), entry );
  gtk_box_pack_start( GTK_BOX( GTK_DIALOG(dialog)->vbox ), entryFrame, TRUE, FALSE, 2 );

  gtk_widget_show_all( entryFrame );
}

void diaObject::run() { doRun(); }

void diaObject::rt() {

  const gchar* strLetter = gtk_entry_get_text( GTK_ENTRY(entry) );
  letter[0] = strLetter[0];
  letter[1] = strLetter[1];
  letter[2] = 0;
}

// diaNorm : dialog class for Norm object -------------------------

diaNorm::diaNorm( const string& _name, char* _letter, bool _bUnite ) : diaObject( _name, _letter ) {

  mlString strUniteButton( IDS_PROPERTIES_UNITE );
  checkbutton = gtk_check_button_new_with_label( ((string)strUniteButton).c_str() );
  gtk_box_pack_start( GTK_BOX( GTK_DIALOG(dialog)->vbox ), checkbutton, TRUE, FALSE, 4 );
  gtk_widget_show( checkbutton );
  bUnite = _bUnite;
  gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( checkbutton ), bUnite );
}

bool diaNorm::run() {

  diaObject::run();
  return bUnite;
}

void diaNorm::rt() {

  diaObject::rt();
  bUnite = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( checkbutton ) );
}

// diaRotated : dialog class for Rotated object ----------------------------

diaRotated::diaRotated( const string& _name, char* _letter, double _angle, double _factor ) : diaObject( _name, _letter ) {

  char conv[128];
  mlString strAngleEdit( IDS_PROPERTIES_ANGLE );
  mlString strFactorEdit( IDS_PROPERTIES_FACTOR );
  angle = _angle*180/M_PI; factor = _factor;

  editAngle = gtk_entry_new_with_max_length( 20 );
  sprintf( conv, "%g", (float)angle );
  gtk_entry_set_text( GTK_ENTRY(editAngle), conv );
  
  editFactor = gtk_entry_new_with_max_length( 20 );
  sprintf( conv, "%g", (float)factor );
  gtk_entry_set_text( GTK_ENTRY(editFactor), conv );
  
  frameAngle = gtk_frame_new( ((string)strAngleEdit).c_str() );
  gtk_container_add( GTK_CONTAINER( frameAngle ), editAngle );

  frameFactor = gtk_frame_new( ((string)strFactorEdit).c_str() );
  gtk_container_add( GTK_CONTAINER( frameFactor ), editFactor );

  hbox = gtk_hbox_new(FALSE,3);
  gtk_box_pack_start( GTK_BOX(hbox), frameAngle, TRUE, FALSE, 3 );
  gtk_box_pack_start( GTK_BOX(hbox), frameFactor, TRUE, FALSE, 3 );

  gtk_box_pack_start( GTK_BOX( GTK_DIALOG(dialog)->vbox ), hbox, TRUE, FALSE, 4 );
  gtk_widget_show_all( hbox );
}

void diaRotated::run() {

  diaObject::run();
}

void diaRotated::rt() {

  diaObject::rt();
  
  const gchar* strConv = gtk_entry_get_text( GTK_ENTRY(editAngle) );
  sscanf( strConv, "%g", &angle );
  strConv = gtk_entry_get_text( GTK_ENTRY(editFactor) );
  sscanf( strConv, "%g", &factor );
}



void diaBary::callbackValueChanged( GtkSpinButton* spinbutton, gpointer user_data ) {

  ((diaBary*)user_data)->valueChanged();
}

diaBary::diaBary( const string& _name, char* _letter, bool _bNPoints, int _nPoints, const double* _weight ) : diaObject( _name, _letter ) {

  mlString strNPointsEdit( IDS_PROPERTIES_NPOINTS );
  mlString strWeightEdit( IDS_PROPERTIES_WEIGHT );
  nPoints = _nPoints;
  bNPoints = _bNPoints;
  for ( int i = 0 ; i < 6 ; i++ ) weight[i] = _weight[i];

  hbox = gtk_hbox_new(FALSE,3);

  for ( int i = 0 ; i < 6 ; i++ ) {
    
    editWeight[i] = gtk_spin_button_new_with_range( -100,100,1 );
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(editWeight[i]), weight[i] );

    frameWeight[i] = gtk_frame_new( ((string)strWeightEdit).c_str() );
    gtk_container_add( GTK_CONTAINER( frameWeight[i] ), editWeight[i] );

   gtk_box_pack_start( GTK_BOX(hbox), frameWeight[i], TRUE, FALSE, 3 );   
  }

  gtk_box_pack_start( GTK_BOX( GTK_DIALOG(dialog)->vbox ), hbox, TRUE, FALSE, 4 );
  gtk_widget_show_all( hbox );

  editNPoints = gtk_spin_button_new_with_range( 1,6,1 );
  gtk_spin_button_set_value( GTK_SPIN_BUTTON(editNPoints), nPoints );

  frameNPoints = gtk_frame_new( ((string)strNPointsEdit).c_str() );
  gtk_container_add( GTK_CONTAINER( frameNPoints ), editNPoints );
  gtk_editable_set_editable( GTK_EDITABLE(editNPoints), bNPoints );
  if ( !bNPoints ) gtk_widget_set_state( editNPoints, GTK_STATE_INSENSITIVE );

  g_signal_connect (GTK_OBJECT (editNPoints),
		    "value-changed",
		    G_CALLBACK (callbackValueChanged),
		    (gpointer)this);  

  gtk_box_pack_start( GTK_BOX( GTK_DIALOG(dialog)->vbox ), frameNPoints, TRUE, FALSE, 4 );
  gtk_widget_show_all( frameNPoints );

  valueChanged();
}

void diaBary::run() {

  diaObject::run();
}

void diaBary::rt() {

  diaObject::rt();
  
  nPoints = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( editNPoints ) );
  for ( int i = 0 ; i < 6 ; i++ ) 
    weight[i] = gtk_spin_button_get_value_as_float( GTK_SPIN_BUTTON( editWeight[i] ) );
}

void diaBary::valueChanged() {

  int nPoints = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( editNPoints ) );
  for ( int i = 0 ; i < 6 ; i++ )
    gtk_widget_set_sensitive( editWeight[i], i<nPoints );
}

// diaColor : dialog class for choosing default colors of objects -----------------------

diaColors::diaColors( const string& _name, GdkColor *_colors, int _nColors, int _strOffset ) : diaBase( _name ) {

  colors = _colors;
  ncolors = _nColors;
  strOffset = _strOffset;
  colorButton = new GtkWidget*[ncolors];

  int ncols = 10;

  GtkWidget* hbox = gtk_hbox_new( TRUE, 3 );

  int j = 0;
  while ( j < ncolors-3 ) {
    
    GtkWidget* vbox = gtk_vbox_new(TRUE, 3 );
    for ( int i=j ; (j < ncolors-3) && (j-i<ncols) ; j++ ) {
      
      mlString str( strOffset+j );
      GtkWidget* frame = gtk_frame_new( ((string)str).c_str() );
      colorButton[j] = gtk_color_button_new_with_color( colors+j );
      gtk_container_add( GTK_CONTAINER( frame ), colorButton[j] );
      gtk_box_pack_start( GTK_BOX( vbox ), frame, TRUE, FALSE, 2 );
    }
    gtk_box_pack_start( GTK_BOX( hbox), vbox, TRUE, FALSE, 2 );
  }
  gtk_box_pack_start( GTK_BOX( GTK_DIALOG(dialog)->vbox ), hbox, TRUE, FALSE, 2 );

  gtk_widget_show_all( dialog );
}

diaColors::~diaColors() {

  delete [] colorButton;
}

void diaColors::run() {

  doRun();
}

void diaColors::rt() {

  for ( int j = 0 ; j < ncolors-3 ; j++ ) {

    gtk_color_button_get_color(GTK_COLOR_BUTTON(colorButton[j]), colors+j);
  }
}

// diaWelcome : window displayed at the beginning of the application --------

#define QUANTUM 10000
// unité : 10 micro-sec
#define BEGIN_WAIT 700
#define END_WAIT 300

diaWelcome::diaWelcome() {

  window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
  gtk_window_set_decorated( GTK_WINDOW( window ), FALSE );
  gtk_window_set_position( GTK_WINDOW( window), GTK_WIN_POS_CENTER_ALWAYS );
  gtk_window_set_keep_above( GTK_WINDOW( window), TRUE );

  GtkWidget* frame = gtk_frame_new( NULL );
  gtk_frame_set_shadow_type(GTK_FRAME( frame ), GTK_SHADOW_OUT );
  gtk_container_set_border_width(GTK_CONTAINER(frame),10);
  gtk_container_add (GTK_CONTAINER(window), frame ); 

  GtkWidget* vbox = gtk_vbox_new(FALSE, 3);
  gtk_container_add (GTK_CONTAINER(frame), vbox );

  string imgLogo = DATADIR; imgLogo += "/img/logo.jpg";
  GtkWidget* image = gtk_image_new_from_file( imgLogo.c_str() );
  gtk_box_pack_start( GTK_BOX(vbox), image, FALSE, FALSE, 0 );  

  gtk_box_pack_start( GTK_BOX(vbox), gtk_hseparator_new(), FALSE, FALSE, 2 );
  
  char text[128]; 
  sprintf( text, "OpenEuclide v%d.%d\n Starting ...", EUCLIDE_MAJOR_VERSION, EUCLIDE_MINOR_VERSION );
  GtkWidget* label = gtk_label_new( text ) ;
  gtk_box_pack_start( GTK_BOX(vbox), label, FALSE, FALSE, 0 );
 
  gtk_widget_show_all( window );

  struct timespec ts;
  ts.tv_sec = 0;
  ts.tv_nsec = QUANTUM;

  for ( long i = 0 ; i < BEGIN_WAIT ; i++ ) {
    
    gtk_main_iteration_do(FALSE);
    nanosleep( &ts, NULL );
  }
}

diaWelcome::~diaWelcome() {

  struct timespec ts;
  ts.tv_sec = 0;
  ts.tv_nsec = QUANTUM;

  for ( long i = 0 ; i < END_WAIT ; i++ ) {
    
    gtk_main_iteration_do(FALSE);
    nanosleep( &ts, NULL );
  }

  gtk_widget_destroy( window );
}
