/*  Copyright 2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "euclideDocument.h"
#include "euclideView.h"
#include "MyArchive.h"
#include "diaBase.h"
#include "callback.h"
#include "id.h"
#include "GeoBase.h"
#include "GeoComment.h"

#include <iostream>
using namespace std;

gint euclideDocument::szX = 1600;
gint euclideDocument::szY = 1200;

euclideDocument::euclideDocument( const string& _name, euclideDocument* sv ) : pln( this, 0, 0 ) {

  name = _name;
  suivant = sv;
  precedent = NULL;
  pixmap = NULL;
  gc = NULL;
  bDragging = FALSE;
  idCompound = 0;
  bAllBlack = FALSE;
  if ( suivant ) suivant->precedent = this;

   // Create widgets

  string title = getFileTitle( name );
  widgetName = gtk_label_new( title.c_str() );
  drawing_area = gtk_drawing_area_new ();

  // Connect the drawing area
  
  gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event",
		      (GtkSignalFunc) _expose_event, this);
  gtk_signal_connect (GTK_OBJECT(drawing_area),"configure_event",
		      (GtkSignalFunc) _configure_event, this);
  gtk_signal_connect (GTK_OBJECT (drawing_area), "motion_notify_event",
		      (GtkSignalFunc) _motion_notify_event, this);
  gtk_signal_connect (GTK_OBJECT (drawing_area), "button_press_event",
		      (GtkSignalFunc) _button_press_event, this);
  gtk_signal_connect (GTK_OBJECT (drawing_area), "button_release_event",
		      (GtkSignalFunc) _button_release_event, this);
  
  gtk_widget_set_events (drawing_area, GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
 			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_POINTER_MOTION_MASK
  			 | GDK_POINTER_MOTION_HINT_MASK);

  gtk_widget_set_size_request(drawing_area, szX, szY );
  gtk_notebook_append_page( GTK_NOTEBOOK( view->noteBook() ), drawing_area, widgetName );
  gtk_widget_show( drawing_area );
  bFileDefined = FALSE;
}

void euclideDocument::load( ifstream& is ) {

  CMyArchive ar;
  ar.fromStream( is );
  if ( !ar.signature( ECL_FILE_VERSION ) ) {

    errBase( IDS_ERR_NOSIGNATURE ).run();
    return;
  }
  pln.Load( ar );
  bFileDefined = TRUE;

  gtk_notebook_set_current_page( GTK_NOTEBOOK(view->noteBook()), myPageIndex() );
  pln.setunmodified();
}

int euclideDocument::myPageIndex() {

  return gtk_notebook_page_num( GTK_NOTEBOOK(view->noteBook()), drawing_area );
}

euclideDocument::~euclideDocument() {

  if ( precedent ) precedent->suivant = suivant;
  else view->detachFirstDocument();
  if ( suivant ) suivant->precedent = precedent;

  gtk_notebook_remove_page( GTK_NOTEBOOK(view->noteBook()), myPageIndex() );
}

bool euclideDocument::preClose() {

  if ( pln.bModified ) {
    bool bError = FALSE;
    mlString strAsk( IDS_ASKFORSAVE );
    errBase dia( name + (string)strAsk );
    if ( dia.run(TRUE) ) save( TRUE, &bError );
    if ( bError ) return FALSE;
  }
  return TRUE;
}

bool euclideDocument::save( bool force, bool* bError ) {

  if ( (!force)&&( !bFileDefined ) ) return FALSE;
  
  ofstream of; of.open( name.c_str(), ios_base::out|ios::binary );
  if ( !of.good() ) {

    errBase err( IDS_CANNOTSAVE );
    err.run(FALSE);
    if ( bError ) *bError = TRUE;
    return TRUE;
  }

  CMyArchive ar;
  pln.Store( ar );
  ar.CloseEvaluating();
  pln.Store( ar );
  ar.CloseStoring();
  ar.toStream( of );
  
  of.close();
  pln.setunmodified();
  return TRUE;
}

bool euclideDocument::saveAs( const string& _name ) {
  
  bFileDefined = TRUE;
  name = _name;
  save();
  return TRUE;
}

euclideDocument* euclideDocument::getFromWidget( GtkWidget* widget ) {
  
  if ( widget == drawing_area ) return this;
  if ( suivant ) return suivant->getFromWidget( widget );
  return NULL;
}

bool euclideDocument::dispatchLetter( char* letter ) {

  if (( letter )&&( letter[1] == 0 )) {
    
    char c = letter[0];
    if ((( c >= 'a' )&&( c <= 'z' ))||(( c >= 'A' )&&( c <= 'Z' ))) {
      pln.DispatchNewLetter( c );
      return TRUE;
    }
  }
  
  return FALSE;
}

gint euclideDocument::configure_event( GtkWidget         *widget,
				       GdkEventConfigure *event ) {

  if (pixmap)
    gdk_pixmap_unref(pixmap);
  
  pixmap = gdk_pixmap_new(widget->window,
                          widget->allocation.width,
                          widget->allocation.height,
                          -1);

  if ( ! gc ) gc = gdk_gc_new(widget->window);

  width = widget->allocation.width;
  height = widget->allocation.height;

  pln.SetScreenFrame( width, height );
  
  return TRUE;
}

gint euclideDocument::expose_event( GtkWidget      *widget,
				    GdkEventExpose *event ) {

  view->lockPainting( TRUE );
  gdk_gc_set_foreground(gc, & colors[black]);
  if ( pixmap ) gdk_draw_pixmap(widget->window,
				widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
				pixmap,
				event->area.x, event->area.y,
				event->area.x, event->area.y,
				event->area.width, event->area.height);

  if ( bDragging ) {
    
    int x1,x2,y1,y2;
    if ( xDrag > x2Drag ) { x2=xDrag; x1=x2Drag; }
    else { x1=xDrag; x2=x2Drag; }
    if ( yDrag > y2Drag ) { y2=yDrag; y1=y2Drag; }
    else { y1=yDrag; y2=y2Drag; }
    gdk_gc_set_foreground( gc, & colors[lightgrey]);
    gdk_gc_set_line_attributes( gc, 1, GDK_LINE_ON_OFF_DASH, GDK_CAP_BUTT, GDK_JOIN_MITER );
    gdk_draw_rectangle( widget->window, gc, FALSE, x1, y1, x2-x1, y2-y1 );
    gdk_gc_set_line_attributes( gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER );
  }
  
  view->lockPainting( FALSE );
  return FALSE;
}

gint euclideDocument::motion_notify_event( GtkWidget       *widget, 
					   gint            x, 
					   gint            y, 
					   GdkModifierType state ) {

  pln.ctrlStatus = ( state & GDK_CONTROL_MASK ) != 0;
  if ( view->bLockPainting() ) return FALSE;
  view->lockPainting(TRUE);
  if ( bDragging ) moveDragging( x, y );
  pln.MouseEvent( evMouseMove, x, y );
  view->lockPainting(FALSE);

  return TRUE; 
}
 
gint euclideDocument::button_press_event( GtkWidget      *widget,
					  GdkEventButton *event ) {

  pln.ctrlStatus = ( event->state & GDK_CONTROL_MASK ) != 0;
  if ( event->type == GDK_BUTTON_PRESS ) {

    if ( event->button == 1 ) {

      if ( ! pln.MouseEvent( evLMouse, event->x, event->y ) ) startDragging( event->x, event->y );
    }
    if ( event->button == 3 ) pln.MouseEvent( evRMouse, event->x, event->y );  
  }
  if ( event->type == GDK_2BUTTON_PRESS ) {

    if ( event->button == 1 ) pln.MouseEvent( evLMouseDblClk, event->x, event->y );
  }
  return TRUE;
}

gint euclideDocument::button_release_event( GtkWidget      *widget,
					    GdkEventButton *event ) {
  
  pln.ctrlStatus = ( event->state & GDK_CONTROL_MASK ) != 0;
  if ( event->button == 1 ) {
    if ( bDragging ) endDragging( event->x, event->y );
    pln.MouseEvent( evLMouseUp, event->x, event->y );
    idCompound = 0;
  }
  if ( event->button == 3 ) pln.MouseEvent( evRMouseUp, event->x, event->y );  
  
  return TRUE;
}

void euclideDocument::createObject( guint idObj ) {
  
  if (( idObj >= IDM_COMPOUND_MIN )&&( idObj <= IDM_COMPOUND_MAX )) 
    idCompound = idObj;
  
  pln.Command( idObj );
}

void euclideDocument::selectAll() {

  pln.selectAll();
}

void euclideDocument::aVoid() {

  gdk_draw_rectangle (pixmap,
	              drawing_area->style->white_gc,
                      TRUE,
                      0, 0,
                      width,height);
}

void euclideDocument::paintPoint( int x, int y, int color, bool bSelected, bool bMark ) {

  if ( bSelected ) {

    gdk_gc_set_foreground(gc, & colors[lightgrey]);
    gdk_draw_arc( pixmap, gc, FALSE, x-6,y-6, 12,12, 0, 360*64 );
  }
  
  if ( bMark ) {

    gdk_gc_set_foreground(gc, & colors[black]);
    gdk_draw_line( pixmap, gc, x-12, y, x-8, y );
    gdk_draw_line( pixmap, gc, x+12, y, x+8, y );
    gdk_draw_line( pixmap, gc, x, y-12, x, y-8 );
    gdk_draw_line( pixmap, gc, x, y+12, x, y+8 );
  }


  gdk_gc_set_foreground( gc, & colors[bAllBlack?black:color]);
  gdk_draw_arc( pixmap, gc, TRUE, x-5,y-5, 10,10, 0, 360*64 );
}

void euclideDocument::paintLetter( int x, int y, int color, char* letter ) {

    gdk_gc_set_foreground(gc, & colors[bAllBlack?black:color]);
    gdk_draw_layout( pixmap, gc, x+1, y+5,
		     gtk_widget_create_pango_layout( drawing_area, letter ) );
    // delete pango layout ?    
}

void euclideDocument::paintLine( Vect& o0, Vect& u0, int color, bool bSelected, bool bMark ) {

  Vect a[2]; int i = 0;
  Vect fr = Vect( width, height, 0 );
  Vect x;
	
  x = o0 - u0*( o0.X[0] / u0.X[0] );
  if (( x.X[1] > 0 )&&( x.X[1] < fr.X[1] )) {
    a[i] = x; i++; }
  
  x = o0 - u0*( ( o0.X[0] - fr.X[0] ) / u0.X[0] );
  if (( x.X[1] > 0 )&&( x.X[1] < fr.X[1] )) {
    a[i] = x; i++; if ( i>1 ) goto thend; }
  
  x = o0 - u0*( o0.X[1] / u0.X[1] );
  if (( x.X[0] > 0 )&&( x.X[0] < fr.X[0] )) {
    a[i] = x; i++; if ( i>1 ) goto thend; }
  
  x = o0 - u0*( ( o0.X[1] - fr.X[1] ) / u0.X[1] );
  if (( x.X[0] > 0 )&&( x.X[0] < fr.X[0] )) {
    a[i] = x; i++; }
  
  if ( i < 2 ) return;
 thend:
  
  if ( ( bSelected || bMark ) ) {
    
    gdk_gc_set_foreground(gc, & colors[lightgrey]);
    gdk_gc_set_line_attributes( gc, 3, (bMark)?(GDK_LINE_ON_OFF_DASH):(GDK_LINE_SOLID), GDK_CAP_BUTT, GDK_JOIN_MITER );
    gdk_draw_line( pixmap, gc, (int)a[0].X[0], (int)a[0].X[1], (int)a[1].X[0], (int)a[1].X[1] );
  }
  
  gdk_gc_set_foreground(gc, & colors[bAllBlack?black:color]);
  gdk_gc_set_line_attributes( gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER );
  gdk_draw_line( pixmap, gc, (int)a[0].X[0], (int)a[0].X[1], (int)a[1].X[0], (int)a[1].X[1] );
}

void euclideDocument::paintSegment( int x1, int y1, int x2, int y2, int color, bool bSelected, bool bMark ) {

    if ( ( bSelected || bMark ) ) {
    
    gdk_gc_set_foreground(gc, & colors[lightgrey]);
    gdk_gc_set_line_attributes( gc, 3, (bMark)?(GDK_LINE_ON_OFF_DASH):(GDK_LINE_SOLID), GDK_CAP_BUTT, GDK_JOIN_MITER );
    gdk_draw_line( pixmap, gc, x1,y1,x2,y2 );
  }
  
  gdk_gc_set_foreground(gc, & colors[bAllBlack?black:color]);
  gdk_gc_set_line_attributes( gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER );
  gdk_draw_line( pixmap, gc, x1,y1,x2,y2 );
}

void euclideDocument::circle( int x, int y, int r, int color, bool bSelect, bool bMark ) {

  if ( bSelect || bMark ) {

    gdk_gc_set_foreground(gc, & colors[lightgrey]);
    gdk_gc_set_line_attributes( gc, 3, (bMark)?(GDK_LINE_ON_OFF_DASH):(GDK_LINE_SOLID), GDK_CAP_BUTT, GDK_JOIN_MITER );
    gdk_draw_arc(pixmap, gc, FALSE, x-r, y-r, 2*r, 2*r, 0, 64*360 );
  }

  gdk_gc_set_foreground(gc, & colors[bAllBlack?black:color]);
  gdk_gc_set_line_attributes( gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER );
  gdk_draw_arc(pixmap, gc, FALSE, x-r, y-r, 2*r, 2*r, 0, 64*360 );
}

void euclideDocument::paintAngle( Vect& o, Vect& p1, Vect& p2, bool bShift, bool bOriented, bool bClockwise, double alpha, int r, int color, bool bSelected, bool bMark ) {
  
  gdk_gc_set_foreground(gc, (bMark)?(& colors[lightgrey]):(& colors[bAllBlack?black:color]));
  gdk_draw_line( pixmap, gc, (int)o.X[0], (int)o.X[1], (int)p1.X[0], (int)p1.X[1] );
  gdk_draw_line( pixmap, gc, (int)o.X[0], (int)o.X[1], (int)p2.X[0], (int)p2.X[1] );
   
  if (( !bOriented )&&( fabs( alpha - M_PI/2 ) < M_PI/256 )) {
    
    // Show orthogonal mark
    
    Vect u1 = 0.5*r * !( p1 - o );
    Vect u2 = 0.5*r * !( p2 - o );
    
    Vect pu1 = o + u1;
    Vect pu2 = o + u2;
    gdk_draw_line( pixmap, gc, (int)pu1.X[0], (int)pu1.X[1], (int)( pu1.X[0] + u2.X[0] ), (int)( pu1.X[1] + u2.X[1] )  );
    gdk_draw_line( pixmap, gc, (int)pu2.X[0], (int)pu2.X[1], (int)( pu2.X[0] + u1.X[0] ), (int)( pu2.X[1] + u1.X[1] )  );
  } else {
    
    int a1 = (bClockwise)?( -(int)(180*64*(p2-o).angle()/M_PI) ):( -(int)(180*64*(p1-o).angle()/M_PI) );
    gdk_draw_arc( pixmap, gc, FALSE, (int)o.X[0]-r, (int)o.X[1]-r, 2*r, 2*r, a1, (int)(64*180*alpha/M_PI) );
    char tmp[40];
    if ( pln.bDegrees ) {
      alpha *= 180/M_PI;
      sprintf( tmp, "%d", (int)alpha );
    } else sprintf( tmp, "%g", alpha );
    
    gdk_draw_layout( pixmap, gc, (int)(o.X[0]+((bShift)?(1.6):(-1))*r), (int)(o.X[1]+1.3*r), 
		     gtk_widget_create_pango_layout( drawing_area, tmp ) );  
  }
}

void euclideDocument::paintComment( int x, int y, int color, const string& str, bool bSelect ) {

  if ( bSelect ) {

    gdk_gc_set_foreground(gc, & colors[lightgrey]);
    gdk_gc_set_line_attributes( gc, 3, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER );
    gdk_draw_line( pixmap, gc, x, y, x+30, y );
    gdk_draw_line( pixmap, gc, x, y, x, y+10 );
  }
   
  gdk_gc_set_foreground(gc, & colors[bAllBlack?black:color]);
  gdk_gc_set_line_attributes( gc, 1, GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER );
  gdk_draw_line( pixmap, gc, x, y, x+30, y );
  gdk_draw_line( pixmap, gc, x, y, x, y+10 );

  gdk_draw_layout( pixmap, gc, x+3, y+3,
		   gtk_widget_create_pango_layout( drawing_area, str.c_str() ) );  
}

void euclideDocument::invalidate() {

  GdkRectangle rect = { 0, 0, width, height };
  gdk_window_invalidate_rect( drawing_area->window, &rect, FALSE );
}

void euclideDocument::undo() {

  pln.Undo();
}

void euclideDocument::redo() {

  pln.Redo();
}

string euclideDocument::getFileTitle( const string& fileName ) {

  int i;
  for ( i = fileName.length()-1 ; ((i> 0)&&(
    (fileName[i]!='/')&&(fileName[i]!='\\')
    )) ; i-- );;
  if ((fileName[i] == '/')||(fileName[i] == '\\' )) i++;
  return fileName.substr( i, fileName.length()-i+1 );
}

void euclideDocument::startDragging( double x, double y ) {

  xDrag = (int)x;  yDrag = (int)y;
  x2Drag = (int)x;  y2Drag = (int)y;
  bDragging = TRUE;
}

void euclideDocument::moveDragging( double x, double y ) {

  x2Drag = (int)x; y2Drag = (int)y;
  pln.UnSelect();
  pln.SelectFromZone( Vect( xDrag, yDrag , 0 ), Vect( x2Drag, y2Drag, 0 ) );
  invalidate();
}

void euclideDocument::endDragging( double x, double y ) {

  if ( idCompound ) createCompound( xDrag, yDrag, (int)x, (int)y );

  bDragging = FALSE;
  invalidate();
}

#define xm ( (x1+x2)/2 )
#define ym ( (y1+y2)/2 )

void euclideDocument::createCompound( int x1, int y1, int x2, int y2 ) {

  if ( x2-x1 < 20 ) x2 = x1+40;
  if ( y2-y1 < 20 ) y2 = y1+40;
  
  switch ( idCompound ) {

  case IDM_TRIANGLE: {
    pln.StoreUndo();
    GeoFixPoint* p1 = new GeoFixPoint( &pln, xm, y1 );
    GeoFixPoint* p2 = new GeoFixPoint( &pln, x1, y2 );
    GeoFixPoint* p3 = new GeoFixPoint( &pln, x2, y2 );
    new GeoLineAB( &pln, p1, p2, TRUE );
    new GeoLineAB( &pln, p1, p3, TRUE );
    new GeoLineAB( &pln, p2, p3, TRUE );
    p1->Letter[0] = 'A';
    p2->Letter[0] = 'B';
    p3->Letter[0] = 'C';
    pln.InvalidateAllOfThem();
    break;
  }
  case IDM_COMP_CARRE: {
    pln.StoreUndo();
    GeoFixPoint* p1 = new GeoFixPoint( &pln, x1, y1 );
    GeoFixPoint* p2 = new GeoFixPoint( &pln, x1, y2 );
    GeoCarre* p3 = new GeoCarre( &pln, p1, p2 );
    GeoVect* p4 = new GeoVect( &pln, p1, p2, p3 );
    new GeoLineAB( &pln, p1, p2, TRUE );
    new GeoLineAB( &pln, p1, p3, TRUE );
    new GeoLineAB( &pln, p2, p4, TRUE );
    new GeoLineAB( &pln, p3, p4, TRUE );
    p1->Letter[0] = 'A';
    p2->Letter[0] = 'B';
    p3->Letter[0] = 'C';
    p4->Letter[0] = 'D';
    pln.InvalidateAllOfThem();
    break;
  }
  case IDM_MARK: {
    pln.StoreUndo();
    GeoFixPoint* o = new GeoFixPoint( &pln, xm, ym );
    GeoFixPoint* p1 = new GeoFixPoint( &pln, x2, ym );
    GeoCarre* c1 = new GeoCarre( &pln, o, p1 );
    new GeoLineAB( &pln, o, p1, FALSE );
    new GeoLineAB( &pln, o, c1, FALSE );
    o->Letter[0] = 'O';
    p1->Letter[0] = 'I';
    c1->Letter[0] = 'J';
    pln.Unite = new GeoNorm( &pln, o, p1 );
    pln.InvalidateAllOfThem();
    break;
  }
  case IDM_TRIANGLE_ORTHOGONAL: {
    pln.StoreUndo();
    GeoFixPoint* A = new GeoFixPoint( &pln, x2, y2 );
    GeoFixPoint* B = new GeoFixPoint( &pln, x1, y2 );
    GeoLineAB* AB = new GeoLineAB( &pln, A, B, TRUE );
    GeoOrthogonal* ort = new GeoOrthogonal( &pln, A, AB );
    ort->bVisible = FALSE;
    GeoFixPoint* C = new GeoFixPoint( &pln, x2, y1 );
    C->isRestricted( ort );
    new GeoLineAB( &pln, B, C, TRUE );
    new GeoLineAB( &pln, A, C, TRUE );
    A->Letter[0] = 'A';
    B->Letter[0] = 'B';
    C->Letter[0] = 'C';
    ( new GeoAngle( &pln, A,B,C ) )->bOriented = FALSE;
    pln.InvalidateAllOfThem();
    break;
  }	
  case IDM_PARALLELOGRAM: {
    pln.StoreUndo();
    GeoFixPoint* A = new GeoFixPoint( &pln, x1, y2 );
    GeoFixPoint* B = new GeoFixPoint( &pln, xm, y2 );
    GeoFixPoint* D = new GeoFixPoint( &pln, xm, y1 );
    GeoVect* C = new GeoVect( &pln, A, B, D );
    new GeoLineAB( &pln, A,B, TRUE );
    new GeoLineAB( &pln, B,C, TRUE );
    new GeoLineAB( &pln, C,D, TRUE );
    new GeoLineAB( &pln, D,A, TRUE );
    A->Letter[0] = 'A';
    B->Letter[0] = 'B';
    C->Letter[0] = 'C';
    D->Letter[0] = 'D';
    pln.InvalidateAllOfThem();
    break;
  }
  case IDM_TRAPEZE: {
    pln.StoreUndo();
    GeoFixPoint* A = new GeoFixPoint( &pln, x1, y2 );
    GeoFixPoint* B = new GeoFixPoint( &pln, x2, y2 );
    GeoFixPoint* D = new GeoFixPoint( &pln, xm, y1 );
    GeoLineAB* AB = new GeoLineAB( &pln, A, B, TRUE );
    GeoParallel* lp = new GeoParallel( &pln, D, AB );
    GeoFixPoint* C = new GeoFixPoint( &pln, x2, y1 );
    C->isRestricted( lp );
    lp->bVisible = FALSE;
    new GeoLineAB( &pln, B,C,TRUE );
    new GeoLineAB( &pln, C,D,TRUE );
    new GeoLineAB( &pln, D,A,TRUE );
    A->Letter[0] = 'A';
    B->Letter[0] = 'B';
    C->Letter[0] = 'C';
    D->Letter[0] = 'D';
    pln.InvalidateAllOfThem();
    break;
  }
  case IDM_TRIANGLE_ISOCEL: {
    pln.StoreUndo();
    GeoFixPoint* A = new GeoFixPoint( &pln, x1, y2 );
    GeoFixPoint* B = new GeoFixPoint( &pln, x2, y2 );
    GeoLineAB* AB = new GeoLineAB( &pln, A, B, TRUE );
    GeoMidPoint* I = new GeoMidPoint( &pln, A, B );
    GeoOrthogonal* ort = new GeoOrthogonal( &pln, I, AB );
    GeoFixPoint* C = new GeoFixPoint( &pln, xm, y1 );
    C->isRestricted( ort );				
    new GeoLineAB( &pln, A,C,TRUE );
    new GeoLineAB( &pln, B,C,TRUE );
    ort->bVisible = FALSE;
    I->bVisible = FALSE;
    A->Letter[0] = 'A';
    B->Letter[0] = 'B';
    C->Letter[0] = 'C';
    pln.InvalidateAllOfThem();
    break;
  }
  case IDM_TRIANGLE_EQUILATERAL: {
    pln.StoreUndo();
    GeoFixPoint* A = new GeoFixPoint( &pln, x1, y2 );
    GeoFixPoint* B = new GeoFixPoint( &pln, x2, y2 );
    GeoPointRotated* C = new GeoPointRotated( &pln, A, B, M_PI/3 );
    new GeoLineAB( &pln, A,C,TRUE );
    new GeoLineAB( &pln, B,C,TRUE );
    new GeoLineAB( &pln, B,A,TRUE );
    A->Letter[0] = 'A';
    B->Letter[0] = 'B';
    C->Letter[0] = 'C';
    pln.InvalidateAllOfThem();
    break;
  }
  case IDM_COMP_CIRCLE: {
    pln.StoreUndo();
    GeoFixPoint* O = new GeoFixPoint( &pln, xm, ym );
    GeoFixPoint* R = new GeoFixPoint( &pln, x2, ym );
    new GeoCircleAB( &pln, O, R );
    O->Letter[0] = 'O';
    R->Letter[0] = 'R';
    pln.InvalidateAllOfThem();
    break;
  }
  case IDM_RECTANGLE: {
    pln.StoreUndo();
    GeoFixPoint* A = new GeoFixPoint( &pln, x1, y2 );
    GeoFixPoint* B = new GeoFixPoint( &pln, x2, y2 );
    GeoLineAB* AB = new GeoLineAB( &pln, A,B,TRUE );
    GeoOrthogonal* lAD = new GeoOrthogonal( &pln, A, AB );
    GeoFixPoint* D = new GeoFixPoint( &pln, x1, y1 );
    D->isRestricted( lAD );
    GeoVect* C = new GeoVect( &pln, A, B, D );
    new GeoLineAB( &pln, A,D,TRUE );
    new GeoLineAB( &pln, B,C,TRUE );
    new GeoLineAB( &pln, C,D,TRUE );
    lAD->bVisible = FALSE;
    A->Letter[0] = 'A';
    B->Letter[0] = 'B';
    C->Letter[0] = 'C';
    D->Letter[0] = 'D';
    pln.InvalidateAllOfThem();
    break;
  }
  }
  idCompound = 0;
}

void euclideDocument::exportAsImage( const gchar* filename, gint xRes, bool _black ) {

  gint oldWidth = width;
  gint oldHeight = height;
  GdkPixmap *oldPixmap = pixmap;

  if ( _black ) bAllBlack = TRUE;

  width = xRes;
  height = (gint)(((double)width * oldHeight) / oldWidth);

  pixmap = gdk_pixmap_new(NULL, width, height, 24);
  pln.UnSelect();
  pln.SetScreenFrame( width, height );

  GdkPixbuf *pixbuf = gdk_pixbuf_get_from_drawable( NULL, pixmap, gdk_drawable_get_colormap(pixmap),
						    0,0,0,0, width, height );
  gdk_pixmap_unref(pixmap);
  width = oldWidth; height = oldHeight; pixmap = oldPixmap;
  pln.SetScreenFrame( width, height );

  gdk_pixbuf_save(pixbuf, filename, "png", NULL, NULL);
  g_object_unref(pixbuf);
  
  bAllBlack = FALSE;
}
